#A* -------------------------------------------------------------------
#B* This file contains source code for the PyMOL computer program
#C* copyright 1998-2000 by Warren Lyford Delano of DeLano Scientific. 
#D* -------------------------------------------------------------------
#E* It is unlawful to modify or remove this copyright notice.
#F* -------------------------------------------------------------------
#G* Please see the accompanying LICENSE file for further information. 
#H* -------------------------------------------------------------------
#I* Additional authors of this source file include:
#-*
#-* NOTE: Based on code by John E. Grayson which was in turn 
#-* based on code written by Doug Hellmann. 
#Z* -------------------------------------------------------------------

# this section is devoted to making sure that Tkinter variables which
# correspond to Menu-displayed settings are kept synchronized with
# PyMOL

from Tkinter import *
import tkColorChooser

import Pmw
from pymol import cmd
import pymol.setting
import string
import copy

class NewColor:
   def __init__(self,app,parent):
      self.app = app
      self.parent = parent
      items = []

      self.dialog = Pmw.PromptDialog(self.app.root,title='Create color named...',
                          buttons = ('Create', 'Cancel'),
                                   defaultbutton='Set',
                          buttonboxpos=S,
                          command = self.command)
      self.dialog.geometry("300x120")
      self.entryfield = self.dialog.component('entryfield')
      self.entry = self.entryfield.component('entry')

      self.entryfield.setentry('')
      self.dialog.protocol('WM_DELETE_WINDOW',self.cancel)

      app.my_activate(self.dialog,focus=self.entry)

   def cancel(self,event=None):
      self.command(result='Done')

   def command(self,result=None):
      if result=='Create':
         st = string.strip(self.entry.get())
         if len(st):
            self.parent.update(st)
         self.app.my_deactivate(self.dialog)
         if len(st):
            ColorEdit(self.app,st,self.parent,[1.0,1.0,1.0])
      else:
         self.app.my_deactivate(self.dialog)
      
class ColorEdit:
   def __init__(self,app,name,parent,rgb):
      self.app = app
      self.parent = parent
      self.name = name

      color = tkColorChooser.Chooser(
         initialcolor='#%02x%02x%02x'%(
         int(rgb[0]*255),int(rgb[1]*255),int(rgb[2]*255)),
         title="Modify color").show()

      if color:
         if color[0]!=None:
            rgb = color[0]
            cmd.do("set_color %s,[%5.2f,%5.2f,%5.2f]"%(name,
                   float(rgb[0]/255.0),
                   float(rgb[1]/255.0),
                   float(rgb[2]/255.0)))
            cmd.do("recolor")
            
class ColorEditor:

   def __init__(self,app):

      self.app = app
      self.list = []

      lst = cmd.get_color_indices()
      for a in lst:
         self.list.append("%-30s"%(a[0]))

      self.index = {}
      c = 0
      for a in lst:
         self.index[a[0]] = a[1]
         c = c + 1
         
      self.dialog = Pmw.SelectionDialog(self.app.root,title="Settings",
                          buttons = ('New', 'Edit', 'Done'),
                                   defaultbutton='Edit',
                          scrolledlist_labelpos=N,
                          label_text='Double click to edit',
                          scrolledlist_items = self.list,
                          command = self.command)
      self.dialog.geometry("500x400")
      self.listbox = self.dialog.component('scrolledlist')
      self.listbox.component('listbox').configure(font=app.my_fw_font)
      self.dialog.protocol('WM_DELETE_WINDOW',self.cancel)
      app.my_show(self.dialog)

   def cancel(self,event=None):
      self.command(result='Done')
      
   def update(self,name):
      if not self.index.has_key(name):
         self.listbox.insert(0,"%s"%name)
         self.listbox.selection_clear()         
         self.listbox.selection_set(0)
         for a in self.index.keys():
            self.index[a]=self.index[a]+1
         self.index[name]=0
      else:
         self.listbox.selection_clear()         
         self.listbox.selection_set(self.index[name])
         
   def command(self,result):
      if result=='Done':
         self.app.my_withdraw(self.dialog)
      elif result=='Edit':
         sels = self.dialog.getcurselection()
         if len(sels)!=0:
            color = string.strip(sels[0])
            ColorEdit(self.app,color,self,cmd.get_color_tuple(color))
      else:
         NewColor(self.app,self)
         



# add your pmg_tk startup scripts to this directory...
# here is an example: (copy, remove comment marks, and save as sample.py)

# BEGIN EXAMPLE: sample.py

#from Tkinter import *
#from pymol import cmd

#def __init__(self):
#   self.menuBar.addmenu('Sample', 'Sample Menu')

#   self.menuBar.addmenuitem('Sample', 'command',
#                      'White background',
#                      label='White Background',
#                     command = lambda : cmd.set("bg_rgb","[1,1,1]"))

#   self.menuBar.addmenuitem('Sample', 'command',
#                      'Black background',
#                      label='Black Background',
#                     command = lambda : cmd.set("bg_rgb","[0,0,0]"))





from chempy import feedback

import string
import copy

default_extra = { # atomic number, and normal valency (for tinker)
   # NOTE: THIS SET IS ONLY USED IF THERE ARE NO TINKER RECORDS
   # IN THE PARAMETER FILE
   # bromine 
   'BR' : [                 35 ,               1 ],
   # carbon                           
   'C'  : [                  6 ,               3 ],
   'CA' : [                  6 ,               3 ],
   'CB' : [                  6 ,               3 ],
   'CC' : [                  6 ,               3 ],
   'CF' : [                  6 ,               3 ],
   'CK' : [                  6 ,               3 ],
   'CM' : [                  6 ,               3 ],
   'CN' : [                  6 ,               3 ],
   'CQ' : [                  6 ,               3 ],
   'CR' : [                  6 ,               3 ],
   'CT' : [                  6 ,               4 ],
   'CV' : [                  6 ,               3 ],
   'CW' : [                  6 ,               3 ], 
   'CY' : [                  6 ,               2 ],
   'CX' : [                  6 ,               2 ],
   'C5' : [                  6 ,               3 ],
   'C*' : [                  6 ,               3 ],
   # calcium
   'C0' : [                 20 ,               3 ],
   # chloride
   'Cl' : [                 17 ,               1 ], 
   # fluorine
   'F'  : [                  9 ,               1 ],
   # hydrogen
   'H'  : [                  1 ,               1 ],
   'H1' : [                  1 ,               1 ],
   'H2' : [                  1 ,               1 ],
   'H3' : [                  1 ,               1 ],
   'H4' : [                  1 ,               1 ],
   'H5' : [                  1 ,               1 ],
   'HA' : [                  1 ,               1 ],
   'HC' : [                  1 ,               1 ],
   'HO' : [                  1 ,               1 ],
   'HP' : [                  1 ,               1 ],
   'HS' : [                  1 ,               1 ],
   'HW' : [                  1 ,               2 ],
   # iodine
   'I'  : [                 53 ,               1 ],
   # chloride anion
   'IM' : [                 17 ,               0 ],
   # sodium cation
   'IP' : [                 11 ,               0 ],
   # magnesium
   'MG' : [                 12 ,               0 ],
   # nitrogen
   'N'  : [                  7 ,               3 ],
   'N*' : [                  7 ,               2 ],
   'N2' : [                  7 ,               3 ],
   'N3' : [                  7 ,               4 ],
   'NA' : [                  7 ,               3 ],
   'NB' : [                  7 ,               2 ],
   'NC' : [                  7 ,               2 ],
   'NX' : [                  7 ,               2 ],
   'NZ' : [                  7 ,               3 ],
   #
   'OW' : [                  8 ,               2 ],
   'OH' : [                  8 ,               2 ],
   'OS' : [                  8 ,               2 ],
   'O'  : [                  8 ,               1 ],
   'O2' : [                  8 ,               1 ],
   'OM' : [                  8 ,               1 ],
   'OZ' : [                  8 ,               1 ],   
   'P'  : [                 15 ,               4 ],
   'S'  : [                 16 ,               2 ],
   'SO'  : [                16 ,               4 ],
   'SX'  : [                16 ,               2 ],
   'SH' : [                 16 ,               2 ],
   # copper
   'CU' : [                 29 ,               0 ],
   # iron
   'FE' : [                 26 ,               0 ],
   # lithium
   'Li' : [                  3 ,               0 ],
   # potassium
   'K'  : [                 19 ,               0 ],
   # rubidium
   'Rb' : [                 37 ,               0 ],
   # cesium
   'Cs' : [                 55 ,               0 ],
   # calcium (alternate)
   'Ca' : [                 20 ,               0 ],
   # dummy
   'X ' : [                  6 ,               1 ],
   # zinc
   'Z0' : [                 30 ,               0 ],
   'Z4' : [                 30 ,               4 ],
   'Z5' : [                 30 ,               5 ],
   }

class Parameters:

   def __init__(self,fname):
      if feedback['actions']:
         print ' '+str(self.__class__)+': loading from "%s"...' % fname
      f = open(fname)
      # skip
      l = f.readline()
      # read names & molecular weights
      self.type = []
      self.mw = {}
      while 1:
         l = string.strip(f.readline())
         if not len(l): break
         a2 = string.strip(l[0:2])
         self.type.append(a2)
         self.mw[a2] = [float(l[3:13]),string.strip(l[34:])]
      # skip 1
      l = f.readline()
      # read bonds
      self.bond = {}
      while 1:
         l = string.strip(f.readline())
         if not len(l): break
         a5 = l[0:5]
         self.bond[a5] = [float(l[5:12]),float(l[12:22]),
                          string.strip(l[22:])]
      # read angles
      self.angle = {}
      while 1:
         l = string.strip(f.readline())
         if not len(l): break
         a5 = l[0:8]
         self.angle[a5] = [float(l[8:16]),float(l[16:28]),
                          string.strip(l[28:])]
      # read torsion 
      self.torsion = {}
      while 1:
         l = string.strip(f.readline())
         if not len(l): break
         a5 = l[0:11]
         if self.torsion.has_key(a5):
            self.torsion[a5].extend(
               [int(l[11:15]),
                float(l[15:27]),
                float(l[27:36]),
                abs(int(float(l[40:52]))),
                string.strip(l[52:])])
         else:
            self.torsion[a5] = [int(l[11:15]),
                                float(l[15:27]),
                                float(l[27:36]),
                                abs(int(float(l[40:52]))),
                                string.strip(l[52:])]
      # read impropers
      self.improper = {}
      while 1:
         l = string.strip(f.readline())
         if not len(l): break
         a5 = l[0:11]
         self.improper[a5] = [float(l[15:27]),
                              float(l[27:40]),
                              abs(int(float(l[40:51]))),
                              string.strip(l[51:])]
      # skip
      while 1:
         l = string.strip(f.readline())
         if not len(l): break
      # read vdw equivalents 
      self.vdw_eq = {}
      while 1:
         l = string.strip(f.readline())
         if not len(l): break
         a4 = string.strip(l[0:4])
         l = l[4:]
         while len(l):
            self.vdw_eq[string.strip(l[0:4])] = a4
            l = l[4:]
      # skip
      l = string.strip(f.readline())
      # read vdw parameters
      self.vdw = {}
      while 1:
         l = string.strip(f.readline())
         if not len(l): break
         l = '  ' + l
         a4 = string.strip(l[0:4])
         self.vdw[a4] =  [float(l[4:20]),
                          float(l[20:37]),
                          string.strip(l[37:])]
         
      # read extra tinker information if present
      self.extra = {}
      while 1:
         l = f.readline()
         if not l: break
         if l[0:6] == 'TINKER':
            self.extra[string.strip(l[6:12])]  = [
               int(l[12:18]),
               int(l[18:24])]
      if not len(self.extra.keys()):
            self.extra = default_extra
      # now generate redundant equivalents
      for a in self.vdw_eq.keys():
         self.vdw[a] = self.vdw[self.vdw_eq[a]]
      for a in self.bond.keys():
         k = a[3:5]+'-'+a[0:2]
         self.bond[k] = self.bond[a]
      for a in self.angle.keys():
         k = a[6:8]+'-'+a[3:5]+'-'+a[0:2]
         self.angle[k] = self.angle[a]
         
   def dump(self):
      for b in self.type:
         print b
      kees = self.mw.keys()
      kees.sort()
      for b in kees:
         print b,self.mw[b]
      kees = self.vdw_eq.keys()
      kees.sort()
      for b in kees:
         print b,self.vdw_eq[b]
      kees = self.vdw.keys()
      kees.sort()
      for b in kees:
         print b,self.vdw[b]
      kees = self.bond.keys()
      kees.sort()
      for b in kees:
         print b,self.bond[b]
      kees = self.angle.keys()
      kees.sort()
      for b in kees:
         print b,self.angle[b]
      kees = self.torsion.keys()
      kees.sort()
      for b in kees:
         print b,self.torsion[b]
      kees = self.improper.keys()
      kees.sort()
      for b in kees:
         print b,self.improper[b]
      kees = self.vdw.keys()
      kees.sort()
      for b in kees:
         print b,self.vdw[b]
         

class Topology:

   def __init__(self,model):
      self.model = model
      if feedback['actions']:
         print ' '+str(self.__class__)+': searching...'
      # get a connected version too
      cmodel = copy.deepcopy(model).convert_to_connected()
      # find atom types in molecule
      self.present = {}
      for a in model.atom:
         self.present[a.text_type] = 1
      # copy bonds
      self.bond = {}
      for b in model.bond:
         a0 = b.index[0]
         a1 = b.index[1]
         if a0 < a1:
            self.bond[(a0,a1)] = 1
         else:
            self.bond[(a1,a0)] = 1
      # find angles
      self.angle = {}
      ang = self.angle
      for b in model.bond:
         a0 = b.index[0]
         a1 = b.index[1]
         for c in cmodel.bond[a0]: # a0 in center
            a2 = c.index[0] 
            if a2 not in (a0,a1): # outside atom
               if a1 < a2:
                  an = (a1,a0,a2)
               else:
                  an = (a2,a0,a1)                  
               ang[an] = 1
            a2 = c.index[1]
            if a2 not in (a0,a1): # outside atom
               if a1 < a2:
                  an = (a1,a0,a2)
               else:
                  an = (a2,a0,a1)                  
               ang[an] = 1
         for c in cmodel.bond[a1]: # a1 in center
            a2 = c.index[0] 
            if a2 not in (a0,a1): # outside atom
               if a0 < a2:
                  an = (a0,a1,a2)
               else:
                  an = (a2,a1,a0)                  
               ang[an] = 1
            a2 = c.index[1]
            if a2 not in (a0,a1): # outside atom
               if a0 < a2:
                  an = (a0,a1,a2)
               else:
                  an = (a2,a1,a0)                  
               ang[an] = 1
      # find torsions
      self.torsion = {}
      tors = self.torsion
      for b in model.bond: # use bond as center of torsion
         a1 = b.index[0]
         a2 = b.index[1]
         for c in cmodel.bond[a1]: 
            a0 = c.index[0] 
            if a0 not in (a1,a2): # outside atom
               for d in cmodel.bond[a2]:
                  a3 = d.index[0] 
                  if a3 not in (a0,a1,a2): # outside atom
                     if a0 < a3:
                        to = (a0,a1,a2,a3)
                     else:
                        to = (a3,a2,a1,a0)                        
                     tors[to] = 1
                  a3 = d.index[1] 
                  if a3 not in (a0,a1,a2): # outside atom
                     if a0 < a3:
                        to = (a0,a1,a2,a3)
                     else:
                        to = (a3,a2,a1,a0)
                     tors[to] = 1
            a0 = c.index[1] 
            if a0 not in (a1,a2): # outside atom
               for d in cmodel.bond[a2]:
                  a3 = d.index[0] 
                  if a3 not in (a0,a1,a2): # outside atom
                     if a0 < a3:
                        to = (a0,a1,a2,a3)
                     else:
                        to = (a3,a2,a1,a0)                        
                     tors[to] = 1
                  a3 = d.index[1] 
                  if a3 not in (a0,a1,a2): # outside atom
                     if a0 < a3:
                        to = (a0,a1,a2,a3)
                     else:
                        to = (a3,a2,a1,a0)                        
                     tors[to] = 1
      # find impropers (only autogenerates for atoms with 3 bonds)
      self.improper = {}
      impr = self.improper
      a2 = 0
      for a in model.atom: # a2 is the center atom 
         bnd = cmodel.bond[a2]
         if len(bnd) == 3:
            lst = []
            for b in bnd:
               at = b.index[0]
               if at == a2:
                  at = b.index[1]
               lst.append(at)
            lst.sort()
            impr[(lst[0],lst[1],a2,lst[2])] = 1
         a2 = a2 + 1
         
      if feedback['actions']:
         print ' '+str(self.__class__)+': found:'
         print ' '+str(self.__class__)+':    types       %6d' % (
            len(self.present.keys()))
         print ' '+str(self.__class__)+':    bonds       %6d' % (
            len(self.bond.keys()))
         print ' '+str(self.__class__)+':    angles      %6d' % (
            len(self.angle.keys()))
         print ' '+str(self.__class__)+':    torsions    %6d' % (
            len(self.torsion.keys()))
         print ' '+str(self.__class__)+':    impropers   %6d' % (
            len(self.improper.keys()))

   def dump(self):
      kees = self.present.keys()
      kees.sort()
      for b in kees:
         print b
      kees = self.bond.keys()
      kees.sort()
      for b in kees:
         print b
      kees = self.angle.keys()
      kees.sort()
      for b in kees:
         print b
      kees = self.torsion.keys()
      kees.sort()
      for b in kees:
         print b
      kees = self.improper.keys()
      kees.sort()
      for b in kees:
         print b

   def get_list(self):
      return [self.bond.keys(),
              self.angle.keys(),
              self.torsion.keys(),
              self.improper.keys()]

class Subset:
   
   def __init__(self,par,top):
      if feedback['actions']:
         print ' '+str(self.__class__)+': applying parameter set to topology...'
      self.model = top.model
      self.present = copy.deepcopy(top.present)
      atype = []
      for a in self.model.atom:
         atype.append(a.text_type)
      # extra tinker info (atomic number, valency)
      self.miss_extra = []
      self.extra = {}
      s_extra = self.extra
      present = top.present
      p_extra = par.extra
      for kee in present.keys():
         if p_extra.has_key(kee):
            s_extra[kee] = p_extra[kee]
         else:
            self.miss_extra.append(kee)
      # molecular weight
      self.miss_mw = []
      self.mw = {}
      s_mw = self.mw
      present = top.present
      p_mw = par.mw
      for kee in present.keys():
         if p_mw.has_key(kee):
            s_mw[kee] = p_mw[kee]
         else:
            self.miss_mw.append(kee)
            
      # van der waals
      self.miss_vdw = []
      self.vdw = {}
      s_vdw = self.vdw
      present = top.present
      p_vdw = par.vdw
      for kee in present.keys():
         if p_vdw.has_key(kee):
            s_vdw[kee] = p_vdw[kee]
         else:
            self.miss_vdw.append(kee)
      # bonds
      self.miss_bond = []
      self.bond = {}
      s_bond = self.bond
      bond = top.bond
      p_bond = par.bond
      for a in bond.keys():
         kee = "%-2s-%-2s" % (atype[a[0]],atype[a[1]])
         if p_bond.has_key(kee):
            s_bond[kee] = p_bond[kee]
         else:
            self.miss_bond.append((a,kee))
      # angles
      self.miss_angle = []
      self.angle = {}
      s_angle = self.angle
      angle = top.angle
      p_angle = par.angle
      for a in angle.keys():
         kee = "%-2s-%-2s-%-2s" % (atype[a[0]],atype[a[1]],atype[a[2]])
         if p_angle.has_key(kee):
            s_angle[kee] = p_angle[kee]
         else:
            self.miss_angle.append((a,kee))
      # torsions      
      self.miss_torsion = []
      self.torsion = {}
      s_torsion = self.torsion
      torsion = top.torsion
      p_torsion = par.torsion
      for a in torsion.keys():
         at0=atype[a[0]]
         at1=atype[a[1]]
         at2=atype[a[2]] # center atom
         at3=atype[a[3]]
         while 1: # not a real loop, just "else" avoidance
            kee1 = "%-2s-%-2s-%-2s-%-2s" % (at0,at1,at2,at3)
            if p_torsion.has_key(kee1):
               s_torsion[kee1] = p_torsion[kee1]
               break
            kee2 = "%-2s-%-2s-%-2s-%-2s" % (at3,at2,at1,at0)
            if p_torsion.has_key(kee2):
               s_torsion[kee2] = p_torsion[kee2]
               break
            kee = "X -%-2s-%-2s-X " % (at1,at2)
            if p_torsion.has_key(kee):
               s_torsion[kee1] = p_torsion[kee]
               break
            kee = "X -%-2s-%-2s-X " % (at2,at1)
            if p_torsion.has_key(kee): 
               s_torsion[kee2] = p_torsion[kee]
               break
            self.miss_torsion.append((a,kee1))
            break
      # impropers      
      self.miss_improper = []
      self.improper = {}
      s_improper = self.improper
      improper = top.improper
      p_improper = par.improper
      for a in improper.keys():
         at0=atype[a[0]]
         at1=atype[a[1]]
         at2=atype[a[2]] # center atom
         at3=atype[a[3]]
         while 1: # not a real loop, just "else" avoidance

            kee1 = "%-2s-%-2s-%-2s-%-2s" % (at1,at3,at2,at0)
            if p_improper.has_key(kee1):
               s_improper[kee1] = p_improper[kee1]
               break
            kee2 = "%-2s-%-2s-%-2s-%-2s" % (at3,at1,at2,at0)
            if p_improper.has_key(kee2):
               s_improper[kee2] = p_improper[kee2]
               break

            kee3 = "%-2s-%-2s-%-2s-%-2s" % (at0,at3,at2,at1)
            if p_improper.has_key(kee3):
               s_improper[kee3] = p_improper[kee3]
               break
            kee4 = "%-2s-%-2s-%-2s-%-2s" % (at3,at0,at2,at1)
            if p_improper.has_key(kee4):
               s_improper[kee4] = p_improper[kee4]
               break

            kee5 = "%-2s-%-2s-%-2s-%-2s" % (at0,at1,at2,at3)
            if p_improper.has_key(kee5):
               s_improper[kee5] = p_improper[kee5]
               break
            kee6 = "%-2s-%-2s-%-2s-%-2s" % (at1,at0,at2,at3)
            if p_improper.has_key(kee6):
               s_improper[kee6] = p_improper[kee6]
               break

            kee = "X -%-2s-%-2s-%-2s" % (at3,at2,at0)
            if p_improper.has_key(kee):
               s_improper[kee1] = p_improper[kee]
               break
            kee = "X -%-2s-%-2s-%-2s" % (at1,at2,at0)
            if p_improper.has_key(kee):
               s_improper[kee2] = p_improper[kee]
               break
            kee = "X -%-2s-%-2s-%-2s" % (at3,at2,at1)
            if p_improper.has_key(kee):
               s_improper[kee3] = p_improper[kee]
               break
            kee = "X -%-2s-%-2s-%-2s" % (at0,at2,at1)
            if p_improper.has_key(kee):
               s_improper[kee4] = p_improper[kee]
               break
            kee = "X -%-2s-%-2s-%-2s" % (at1,at2,at3)
            if p_improper.has_key(kee):
               s_improper[kee5] = p_improper[kee]
               break
            kee = "X -%-2s-%-2s-%-2s" % (at0,at2,at3)
            if p_improper.has_key(kee):
               s_improper[kee6] = p_improper[kee]
               break

            kee = "X -X -%-2s-%-2s" % (at2,at0)
            if p_improper.has_key(kee):
               s_improper[kee1] = p_improper[kee]
               break
            kee = "X -X -%-2s-%-2s" % (at2,at1)
            if p_improper.has_key(kee):
               s_improper[kee3] = p_improper[kee]
               break
            kee = "X -X -%-2s-%-2s" % (at2,at3)
            if p_improper.has_key(kee):
               s_improper[kee5] = p_improper[kee]
               break
            self.miss_improper.append((a,kee1))
            break

      if feedback['actions']:
         print ' '+str(self.__class__)+': missing:'
         print ' '+str(self.__class__)+':    mol. wts.         %6d' % (
            len(self.miss_mw))
         print ' '+str(self.__class__)+':    vdw               %6d' % (
            len(self.miss_vdw))
         print ' '+str(self.__class__)+':    bonds             %6d' % (
            len(self.miss_bond))
         print ' '+str(self.__class__)+':    angles            %6d' % (
            len(self.miss_angle))
         print ' '+str(self.__class__)+':    torsions          %6d' % (
            len(self.miss_torsion))
         print ' '+str(self.__class__)+':    impropers         %6d (usually okay)' % (
            len(self.miss_improper))
         print ' '+str(self.__class__)+':    extra tinker info %6d' % (
            len(self.miss_extra))
      
   def dump(self):
      kees = self.mw.keys()
      kees.sort()
      for b in kees:
         print b,self.mw[b]
      kees = self.bond.keys()
      kees.sort()
      for b in kees:
         print b,self.bond[b]
      kees = self.angle.keys()
      kees.sort()
      for b in kees:
         print b,self.angle[b]
      kees = self.torsion.keys()
      kees.sort()
      for b in kees:
         print b,self.torsion[b]
      kees = self.improper.keys()
      kees.sort()
      for b in kees:
         print b,self.improper[b]
      kees = self.vdw.keys()
      kees.sort()
      for b in kees:
         print b,self.vdw[b]

   def dump_missing(self,impropers=0):
      if len(self.miss_mw):
         print ' '+str(self.__class__)+': missing molecular weights...'
      for b in self.miss_mw:
         print " ",b
      if len(self.miss_vdw):
         print ' '+str(self.__class__)+': missing van der Waalss...'
      for b in self.miss_vdw:
         print " ",b
      if len(self.miss_bond):
         print ' '+str(self.__class__)+': missing bonds...'
      for b in self.miss_bond:
         print " ",b
      if len(self.miss_angle):
         print ' '+str(self.__class__)+': missing angless...'
      for b in self.miss_angle:
         print " ",b
      if len(self.miss_torsion):
         print ' '+str(self.__class__)+': missing torsions...'
      for b in self.miss_torsion:
         print " ",b
      if impropers:
         if len(self.miss_improper):
            print ' '+str(self.__class__)+': missing impropers...'
         for b in self.miss_improper:
            print " ",b

   def complete(self):
      if not ( len(self.miss_mw) or
               len(self.miss_vdw) or
               len(self.miss_bond) or
               len(self.miss_angle) or
               len(self.miss_torsion) ):
         return 1
      else:
         return 0

   def assign_types(self):
      c = 0
      self.mapping = {}
      map = self.mapping
      kees = self.present.keys()
      label = []
      type = []
      kees.sort()
      for a in kees:
         c = c + 1
         st = str(c)
         label.append(st)
         type.append(a)
         map[a] = st
      # assign numeric types 
      for a in self.model.atom:
         a.numeric_type = map[a.text_type]
      
   def write_tinker_prm(self,fname,proofread=None,smooth=None):
      c = 0
      self.mapping = {}
      map = self.mapping
      kees = self.present.keys()
      label = []
      type = []
      kees.sort()
      if proofread:
         for a in kees:
            label.append(a)
            type.append(a)
            map[a] = a
      else:
         for a in kees:
            c = c + 1
            st = str(c)
            label.append(st)
            type.append(a)
            map[a] = st
      # assign numeric types as per the parameter we're writing
      for a in self.model.atom:
         a.numeric_type = map[a.text_type]
      f = open(fname,'w')
      if smooth:
         f.write('''
forcefield              SMOOTH AMBER
vdwtype                 GAUSSIAN
gausstype               LJ-2
''')
      else:
         f.write('''
forcefield              AMBER95
vdwtype                 LENNARD-JONES
''')
      f.write('''
radiusrule              ARITHMETIC
radiustype              R-MIN
radiussize              RADIUS
epsilonrule             GEOMETRIC
vdw-14-use
vdw-scale               2.0
chg-14-use
chg-scale               1.2
dielectric              1.0
''')

      # types
      for c in range(len(self.present)):
         at = type[c]
         f.write("atom %6s %6s    %-2s      %-25s %3d %10.3f%6d\n" % (
            label[c],label[c],at,'"'+self.mw[at][1][0:23]+'"',
            self.extra[at][0],
            self.mw[at][0],self.extra[at][1]))
      # van der waals
      for c in range(len(self.present)):
         at = type[c]
         f.write("vdw   %8s          %10.4f %10.4f\n" % (
            map[at],self.vdw[at][0],self.vdw[at][1]))
      # bonds
      bond = {}
      for a in self.bond.keys():
         kee = (map[string.strip(a[0:2])],map[string.strip(a[3:5])])
         bond[kee] = a
      kees = bond.keys()
      kees.sort()
      for a in kees:
         kee = bond[a]
         f.write("bond    %6s%5s     %10.1f %10.4f\n" % (
            a[0],a[1],self.bond[kee][0],self.bond[kee][1]))
      # angles
      angle = {}
      for a in self.angle.keys():
         kee = (map[string.strip(a[0:2])],
                map[string.strip(a[3:5])],
                map[string.strip(a[6:8])],                
                )
         angle[kee] = a
      kees = angle.keys()
      kees.sort()
      for a in kees:
         kee = angle[a]
         f.write("angle    %5s%5s%5s%9.2f    %8.2f\n" % (
            a[0],a[1],a[2],self.angle[kee][0],self.angle[kee][1]))
      # impropers
      if not smooth:
         improper = {}
         for a in self.improper.keys():
            kee = (map[string.strip(a[0:2])],
                   map[string.strip(a[3:5])],
                   map[string.strip(a[6:8])],
                   map[string.strip(a[9:11])],                                
                   )
            improper[kee] = a
         kees = improper.keys()
         kees.sort()
         for a in kees:
            kee = improper[a]
            f.write("imptors  %5s%5s%5s%5s     %10.3f%7.1f%3d\n" % (
               a[0],a[1],a[2],a[3],self.improper[kee][0],
               self.improper[kee][1],self.improper[kee][2]))
      else:
         improper = {}
         for a in self.improper.keys():
            kee = (map[string.strip(a[0:2])],
                   map[string.strip(a[3:5])],
                   map[string.strip(a[6:8])],
                   map[string.strip(a[9:11])],                                
                   )
            improper[kee] = a
         kees = improper.keys()
         kees.sort()
         for a in kees:
            kee = improper[a]
            f.write("improper %5s%5s%5s%5s      %10.2f       0.00\n" % (
               a[2],a[0],a[1],a[3],self.improper[kee][0]*3.5))
      # torsions
      torsion = {}
      for a in self.torsion.keys():
         kee = (map[string.strip(a[0:2])],
                map[string.strip(a[3:5])],
                map[string.strip(a[6:8])],
                map[string.strip(a[9:11])],                                
                )
         torsion[kee] = a
      kees = torsion.keys()
      kees.sort()
      for a in kees:
         kee = torsion[a]
         st = "torsion  %5s%5s%5s%5s     " % (
            a[0],a[1],a[2],a[3])
         lst = self.torsion[kee]
         div = lst[0]
         lst = lst[1:]
         while len(lst):
            st = st + "%10.3f%7.1f%3d  " % (
               lst[0]/div,lst[1],lst[2])
            lst = lst[5:]
         while len(st)>79:
            st = string.replace(st,'  ',' ')
         f.write(st+"\n")
      # null charge records
      for c in range(len(self.present)):
         a = label[c]
         f.write("charge  %6s    %10.4f\n" % (a,0.0))
      f.close()


   def get_list(self):
      list = []
      c = 0
      self.mapping = {}
      map = self.mapping
      kees = self.present.keys()
      label = []
      type = []
      kees.sort()
      for a in kees:
         c = c + 1
         st = str(c)
         type.append(a)
         map[a] = c
      # assign numeric types as per the parameter we're writing
      for a in self.model.atom:
         a.numeric_type = map[a.text_type]
      # types
      typ_list = []
      for c in range(len(self.present)):
         at = type[c]
         typ_list.append((map[at],self.extra[at][0],self.mw[at][0],self.extra[at][1]))
      list.append(typ_list)
      # van der waals
      vdw_list = []
      for c in range(len(self.present)):
         at = type[c]
         vdw_list.append((map[at],self.vdw[at][0],self.vdw[at][1]))
      list.append(vdw_list)
      # bonds
      bnd_list = []
      bond = {}
      for a in self.bond.keys():
         kee = (map[string.strip(a[0:2])],map[string.strip(a[3:5])])
         bond[kee] = a
      kees = bond.keys()
      kees.sort()
      for a in kees:
         kee = bond[a]
         bnd_list.append((a[0],a[1],self.bond[kee][0],self.bond[kee][1]))
      list.append(bnd_list)
      # angles
      ang_list = []
      angle = {}
      for a in self.angle.keys():
         kee = (map[string.strip(a[0:2])],
                map[string.strip(a[3:5])],
                map[string.strip(a[6:8])],                
                )
         angle[kee] = a
      kees = angle.keys()
      kees.sort()
      for a in kees:
         kee = angle[a]
         ang_list.append((a[0],a[1],a[2],self.angle[kee][0],self.angle[kee][1]))
      list.append(ang_list)
      # impropers
      imp_list = []
      improper = {}
      for a in self.improper.keys():
         kee = (map[string.strip(a[0:2])],
                map[string.strip(a[3:5])],
                map[string.strip(a[6:8])],
                map[string.strip(a[9:11])],                                
                )
         improper[kee] = a
      kees = improper.keys()
      kees.sort()
      for a in kees:
         kee = improper[a]
         imp_list.append((a[0],a[1],a[2],a[3],self.improper[kee][0],
            self.improper[kee][1],self.improper[kee][2]))
      list.append(imp_list)
      # torsions
      tor_list = []
      torsion = {}
      for a in self.torsion.keys():
         kee = (map[string.strip(a[0:2])],
                map[string.strip(a[3:5])],
                map[string.strip(a[6:8])],
                map[string.strip(a[9:11])],                                
                )
         torsion[kee] = a
      kees = torsion.keys()
      kees.sort()
      for a in kees:
         kee = torsion[a]
         tor = [ a[0],a[1],a[2],a[3] ]
         lst = self.torsion[kee]
         div = lst[0]
         lst = lst[1:]
         while len(lst):
            tor.extend([lst[0]/div,lst[1],lst[2]])
            lst = lst[5:]
         tor_list.append(tor)
      list.append(tor_list)
      return list




from chempy import bmin,feedback
from chempy import io

import os
import re
import getpass # for getuser()

class State:

   def __init__(self):
      if feedback['verbose']:
         print ' '+str(self.__class__)+': created.'
      self.default = {}
      self.echo = 0
      self.model = None
      self.counter = 0
      self.prefix = "bmintmp"
      
   def minimize(self,max_iter=100,fix_flag=None,rest_flag=None,
                rest_coeff = 100.0,solvation=None):
      if feedback['actions']:
         print ' '+str(self.__class__)+': starting minimization run...'
      io.mmd.toFile(self.model,self.prefix+".dat")
      
      f = open(self.prefix+".com",'w')
      # get home-relative path
      pth = os.getcwd()
      pth = re.sub(r".*\/"+getpass.getuser()+"\/",'',pth)
      # provide io filenames
      f.write("%s\n%s\n"%(pth+"/"+self.prefix+".dat",
                          pth+"/"+self.prefix+".out"))
      f.write(" MMOD       0      1      0      0     0.0000     0.0000     0.0000     0.0000\n")
      # select forcefield treatments
      if not solvation: # no solvent, constant dielectric
         f.write(" FFLD      10      1      0      1     1.0000     0.0000     0.0000     0.0000\n")
      else: 
         f.write(''' FFLD      10      1      0      1     1.0000     0.0000     0.0000     0.0000
 SOLV       3      1      0      0     0.0000     0.0000     0.0000     0.0000
 EXNB       0      0      0      0     0.0000     0.0000     0.0000     0.0000
''')
      # read files
      f.write(" READ       0      0      0      0     0.0000     0.0000     0.0000     0.0000\n")
      # fix/restrain atoms according to flags provided
      if fix_flag!=None: # are we fixing any atoms?
         c = 0
         mask= 2 ** fix_flag
         for a in self.model.atom:
            c = c + 1
            if mask&a.flags:
               f.write(" FXAT  %6d      0      0      0    -1.0000     0.0000     0.0000     0.0000\n"%
                       c)
      if rest_flag!=None: # are we restraining any atoms?
         c = 0
         mask= 2 ** rest_flag
         for a in self.model.atom:
            c = c + 1
            if mask&a.flags:
               f.write(" FXAT  %6d      0      0      0 %10.4f     0.0000     0.0000     0.0000\n"%
                       (c,rest_coeff))
      f.write(
''' CONV       2      0      0      0     0.0500     0.0000     0.0000     0.0000
 MINI       1      0 %6d      0     0.0000     0.0000     0.0000     0.0000
 '''%(max_iter))
      f.close()
      
      bmin.do(self.prefix)
      io.mmd.updateFromFile(self.model,self.prefix+".out")
      if hasattr(self.model.molecule,'energy'):
         self.model.molecule.title = "%1.3f"%self.model.molecule.energy
   def load_model(self,a):
      if feedback['verbose']:
         print ' '+str(self.__class__)+': new model loaded.'
      self.model = a









import chempy
import copy
from chempy.models import Indexed

from Numeric import *
from Precision import *

class FastModel:

#------------------------------------------------------------------------------
   def __init__(self):
      self.reset()

#------------------------------------------------------------------------------
   def reset(self):
      self.nAtom = 0
      self.molecule = chempy.Molecule()
      self.txta = None
      self.inta = None
      self.flta = None
      self.crda = None
      self.bnda = None

#------------------------------------------------------------------------------
   def from_indexed(self,model):
      self.reset()
      self.nAtom = model.nAtom
      self.nBond = model.nBond
      self.molecule = copy.deepcopy(model.molecule)
      self.txta = resize(array(' ','c'),(self.nAtom,as_width))
      self.inta = zeros((self.nAtom,ai_width),Int32)
      self.flta = zeros((self.nAtom,af_width),Float)
      self.bnda = zeros((self.nBond,bi_width),Int32)
      self.crda = zeros((self.nAtom,3),Float)

      c = 0
      for a in model.atom:
         txt = "%-4s%-2s%-1s%-4s%-1s%-4s%-1s%-4s%-20s" % \
               (a.name[0:4],a.symbol[0:2],a.alt[0:1],a.resn[0:4],
                a.resn_code[0:1],a.resi[0:4],a.chain[0:1],
                a.segi[0:4],a.text_type[0:20])
         self.txta[c] = txt
         self.inta[c] = [ a.resi_number, a.hetatm, a.formal_charge,
                          a.flags, a.color_code, a.stereo, a.numeric_type ]
         self.flta[c] = [ a.b, a.q, a.partial_charge, a.vdw ]
         self.crda[c] = [ a.coord[0], a.coord[1], a.coord[2] ]
         c = c + 1

      c = 0
      for b in model.bond:
         self.bnda[c] = [ b.index[0], b.index[1], b.order, b.stereo ]
         c = c + 1

#------------------------------------------------------------------------------
   def convert_to_indexed(self):
      
      model = Indexed()
      model.molecule = copy.deepcopy(self.molecule)
      
      for c in xrange(self.nAtom):
         at = chempy.Atom()
         txta = self.txta[c]

         for attrib in ( 'name', 'symbol', 'resn', 'resn_code', 'resi',
                         'alt', 'chain', 'segi', 'text_type' ):
            ll = as[attrib]
            setattr(at,attrib,string.strip(string.join(txta[ll[0]:ll[1]],'')))

         inta = self.inta[c]
         for attrib in ( 'resi_number', 'hetatm', 'formal_charge','flags',
                         'color_code', 'stereo', 'numeric_type' ):
            setattr(at,attrib,inta[ai[attrib]])

         flta = self.flta[c]
         for attrib in ( 'b', 'q', 'partial_charge' ) :
            setattr(at,attrib,flta[af[attrib]])

         crda = self.crda[c]
         at.coord = [crda[0],crda[1],crda[2]]

         # probably need to add some checking here to eliminate values
         # which come back as defaults
         
         model.atom.append(at)
         
      for c in xrange(self.nBond):
         bnd = chempy.Bond()
         bnda = self.bnda[c]
         bnd.index = [bnda[bi_index0],bnda[bi_index1]]
         bnd.order = bnda[bi_order]
         bnd.stereo = bnda[bi_stereo]
         model.bond.append(bnd)

      return model
   
#------------------------------------------------------------------------------

# text properties

as = {}
as_width = 0

as['name'] = [ as_width ]
as_width = as_width + 4
as['name'].append(as_width)

as['symbol'] = [ as_width ]
as_width = as_width + 2
as['symbol'].append(as_width)

as['alt'] = [ as_width ]
as_width = as_width + 1
as['alt'].append(as_width)

as['resn'] = [ as_width ]
as_width = as_width + 4
as['resn'].append(as_width)

as['resn_code'] = [ as_width ]
as_width = as_width + 1
as['resn_code'].append(as_width)

as['resi'] = [ as_width ]
as_width = as_width + 4
as['resi'].append(as_width)

as['chain'] = [ as_width ]
as_width = as_width + 1
as['chain'].append(as_width)

as['segi' ] = [ as_width ]
as_width = as_width + 4
as['segi'].append(as_width)

as['text_type'] = [ as_width ]
as_width = as_width + 20
as['text_type'].append(as_width)

# integer properties

ai = {}
ai_width = 0
ai['resi_number']    = ai_width
ai_width = ai_width + 1
ai['hetatm']         = ai_width
ai_width = ai_width + 1
ai['formal_charge']  = ai_width
ai_width = ai_width + 1
ai['flags']          = ai_width
ai_width = ai_width + 1
ai['color_code']     = ai_width
ai_width = ai_width + 1
ai['stereo']         = ai_width
ai_width = ai_width + 1
ai['numeric_type']   = ai_width
ai_width = ai_width + 1

# float properties

af = {}
af_width = 0
af['b']              = af_width
af_width = af_width + 1
af['q']              = af_width
af_width = af_width + 1
af['partial_charge'] = af_width
af_width = af_width + 1
af['vdw'] = af_width
af_width = af_width + 1

# bond information

bi_index0         = 0
bi_index1         = 1
bi_order          = 2
bi_stereo         = 3
bi_width          = 4





# $Id: __init__.py,v 1.1 2001/02/24 12:11:45 wdelano Exp $
import sys
from pymol import opengl

if opengl._numeric:
    from Numeric import ArrayType
    try:
        import _glu_num
        _glu = _glu_num
    except ImportError:
        import _glu
    except SystemError:
        import _glu
else:
    import _glu

from gluconst import *
from pymol.opengl.gl import Error, CarefulFunction

origdict = _glu.__dict__.copy()
sys.modules['pymol.opengl.glu'].__dict__.update(origdict)

carefuldict = {}
for name, func in origdict.items():
    if callable(func):
        carefuldict[name] = CarefulFunction(name, func)
# These do the same sorts of things that the C versions would

def careful():
    cd = carefuldict.copy()
    if cd.has_key('error'): del cd['error']
    sys.modules['pymol.opengl.glu'].__dict__.update(cd)

import string
def fast():
    cd = origdict.copy()
    if cd.has_key('error'): del cd['error']
    if cd.has_key('glconst'): del cd['glconst']
    sys.modules['pymol.opengl.glu'].__dict__.update(cd)



""" an XML-RPC server to allow remote control of PyMol

  Author: Greg Landrum (Landrum@RationalDiscovery.com)
  Created:  January 2002
  License:  PyMol
  Requires:
            - a python xmlrpclib distribution containing the SimpleXMLRPCServer
              module (1.0 or greater should be fine)
            - python with threading enabled  
             
            
"""
import SimpleXMLRPCServer
import threading
from pymol import cmd,cgo

# initial port to try for the server
_xmlPort=9123
# number of alternate ports to try if the first fails
_nPortsToTry=5
def rpcCmd(cmdText):
  """ executes a PyMol API command

   return value is either the result of the command or the empty string

  """
  res = cmd.do(cmdText)
  if res is not None:
    return res
  else:
    return ''

def rpcLabel(pos,labelText,color=(1,1,1),id='lab1'):
  """ create a text label

    Arguments:
      pos: a 3 tuple with the position of the label
      text: a string with the label
      color: a 3 tuple with the color of the label. (1,1,1) is white
      id: (OPTIONAL) the name of the object to be created

    NOTE:
      at the moment this is, how you say, a hack
      Also at the moment the color argument is ignored

  """
  # FIX: this don't work
  r,g,b = color
  x,y,z = pos

  text = "HETATM%5d  C   UNK     1    %8.3f%8.3f%8.3f  1.00 10.00\n"%(1,x,y,z),
  cmd.read_pdbstr(text,id)
  cmd.label("(%s)"%(id),labelText)
  cmd.hide("nonbonded",id)
  return 1


def rpcSphere(pos,rad,color,id='cgo',extend=1):
  """ create a sphere

    Arguments:
      pos: a 3 tuple with the position of the sphere
      rad: a float with the radius
      color: a 3 tuple with the color of the sphere. (1,1,1) is white
      id: (OPTIONAL) the name of the object to be created
      extend: (OPTIONAL) if this is nonzero, the object will be cleared
        before adding the new sphere.  Otherwise the sphere is appended
        to the ojbect

  """
  r,g,b = color
  x,y,z = pos
  if extend:
    obj = cgoDict.get(id,[])
  else:
    obj = []
  obj.extend([cgo.COLOR,r,g,b,cgo.SPHERE,x,y,z,rad])
  cgoDict[id] = obj
  cmd.load_cgo(obj,id,1)
  return 1


def rpcCylinder(end1,end2,rad,color1,id='cgo',color2=None,extend=1):
  """ create a cylinder

    Arguments:
      end1: a 3 tuple with the position of end1 of the sphere
      end2: a 3 tuple with the position of end1 of the sphere
      rad: a float with the radius
      color1: a 3 tuple with the color of end1 of the sphere. (1,1,1) is white
      id: (OPTIONAL) the name of the object to be created
      color2: (OPTIONAL) a 3 tuple with the color of end2 of the sphere. (1,1,1) is white
      extend: (OPTIONAL) if this is nonzero, the object will be cleared
        before adding the new sphere.  Otherwise the sphere is appended
        to the ojbect

    NOTE: the reason that color2 follows id is that I think clients are
    going to be interested in setting the id more often than they are going
    to care about the second color.
    
  """
  global cgoDict
  
  if color2 is None: color2 = color1
  r1,g1,b1 = color1
  r2,g2,b2 = color2
  x1,y1,z1 = end1
  x2,y2,z2 = end2
  if extend:
    obj = cgoDict.get(id,[])
  else:
    obj = []
  obj.extend([cgo.CYLINDER,x1,y1,z1,x2,y2,z2,rad,r1,g1,b1,r2,g2,b2,])
  cgoDict[id] = obj
  cmd.load_cgo(obj,id,1)
  return 1

def launch_XMLRPC(hostname='localhost',port=_xmlPort,nToTry=_nPortsToTry):
  """ launches the xmlrpc server into a separate thread

    Arguments:
      hostname: name of the host for the server
      port: (OPTIONAL) the first port to try for the server
      nToTry: (OPTIONAL) the number of possible ports to try
        (in case the first can't be opened)

  """
  global cgoDict
  cgoDict = {}
  for i in range(nToTry):
    try:
      serv = SimpleXMLRPCServer.SimpleXMLRPCServer((hostname,port+i))
    except:
      serv = None
    else:
      break
  if serv:
    print 'xml-rpc server running on port %d'%(port+i)
    serv.register_function(rpcCmd,'do')
    serv.register_function(rpcSphere,'sphere')
    serv.register_function(rpcCylinder,'cylinder')
    #serv.register_function(rpcLabel,'label')
    t = threading.Thread(target=serv.serve_forever)
    t.setDaemon(1)
    t.start()
  else:
    print 'xml-rpc server could not be started'



# pymol

from chempy import Bond
import copy
import whrandom

# cap gaps in chain as aldehydes and neutral amines...
# (for instance, when

def cap(object):
   from pymol import cmd
   
   model = cmd.get_model(object)
   # guarantee identical ordering
   cmd.delete(object)
   cmd.load_model(model,object)
   n_list = cmd.identify("(n;n &!(n;c a;2.0))")
   c_list = cmd.identify("(n;c &!(n;n a;2.0))")
   print n_list
   print c_list
   for a in n_list:
      newat = copy.deepcopy(model.atom[a])
      newat.coord = [
         newat.coord[0] + whrandom.random(),
         newat.coord[1] + whrandom.random(),
         newat.coord[2] + whrandom.random(),
         ]
      newat.symbol = 'H'
      newat.name = 'HN'
      newat.numeric_type = 43
      bond = Bond()
      bond.order = 1
      bond.stereo = 0
      bond.index = [ a, model.nAtom ]
      print "adding",newat.name,bond.index
      model.add_atom(newat)
      model.add_bond(bond)
   for a in c_list:
      newat = copy.deepcopy(model.atom[a])
      newat.coord = [
         newat.coord[0] + whrandom.random(),
         newat.coord[1] + whrandom.random(),
         newat.coord[2] + whrandom.random(),
         ]
      newat.symbol = 'H'
      newat.name = 'HC'
      newat.numeric_type = 41
      bond = Bond()
      bond.order = 1
      bond.stereo = 0
      bond.index = [ a, model.nAtom ]
      print "adding",newat.name,bond.index
      model.add_atom(newat)
      model.add_bond(bond)
   # reload
   cmd.delete(object)
   cmd.load_model(model,object)
   cmd.sort(object)
   




# Generated by h2py from glu.h
from pymol.opengl.gl.glconst import *

# Included from GL/gl.h
GLU_VERSION_1_1 = 1
GLU_VERSION_1_2 = 1
GLU_INVALID_ENUM = 100900
GLU_INVALID_VALUE = 100901
GLU_OUT_OF_MEMORY = 100902
GLU_VERSION = 100800
GLU_EXTENSIONS = 100801
GLU_SMOOTH = 100000
GLU_FLAT = 100001
GLU_NONE = 100002
GLU_POINT = 100010
GLU_LINE = 100011
GLU_FILL = 100012
GLU_SILHOUETTE = 100013
GLU_OUTSIDE = 100020
GLU_INSIDE = 100021
GLU_TESS_MAX_COORD = 1.0e150
GLU_TESS_WINDING_RULE = 100140
GLU_TESS_BOUNDARY_ONLY = 100141
GLU_TESS_TOLERANCE = 100142
GLU_TESS_WINDING_ODD = 100130
GLU_TESS_WINDING_NONZERO = 100131
GLU_TESS_WINDING_POSITIVE = 100132
GLU_TESS_WINDING_NEGATIVE = 100133
GLU_TESS_WINDING_ABS_GEQ_TWO = 100134
GLU_TESS_BEGIN = 100100
GLU_TESS_VERTEX = 100101
GLU_TESS_END = 100102
GLU_TESS_ERROR = 100103
GLU_TESS_EDGE_FLAG = 100104
GLU_TESS_COMBINE = 100105
GLU_TESS_BEGIN_DATA = 100106
GLU_TESS_VERTEX_DATA = 100107
GLU_TESS_END_DATA = 100108
GLU_TESS_ERROR_DATA = 100109
GLU_TESS_EDGE_FLAG_DATA = 100110
GLU_TESS_COMBINE_DATA = 100111
GLU_TESS_ERROR1 = 100151
GLU_TESS_ERROR2 = 100152
GLU_TESS_ERROR3 = 100153
GLU_TESS_ERROR4 = 100154
GLU_TESS_ERROR5 = 100155
GLU_TESS_ERROR6 = 100156
GLU_TESS_ERROR7 = 100157
GLU_TESS_ERROR8 = 100158
GLU_TESS_MISSING_BEGIN_POLYGON = GLU_TESS_ERROR1
GLU_TESS_MISSING_BEGIN_CONTOUR = GLU_TESS_ERROR2
GLU_TESS_MISSING_END_POLYGON = GLU_TESS_ERROR3
GLU_TESS_MISSING_END_CONTOUR = GLU_TESS_ERROR4
GLU_TESS_COORD_TOO_LARGE = GLU_TESS_ERROR5
GLU_TESS_NEED_COMBINE_CALLBACK = GLU_TESS_ERROR6
GLU_AUTO_LOAD_MATRIX = 100200
GLU_CULLING = 100201
GLU_SAMPLING_TOLERANCE = 100203
GLU_DISPLAY_MODE = 100204
GLU_PARAMETRIC_TOLERANCE = 100202
GLU_SAMPLING_METHOD = 100205
GLU_U_STEP = 100206
GLU_V_STEP = 100207
GLU_PATH_LENGTH = 100215
GLU_PARAMETRIC_ERROR = 100216
GLU_DOMAIN_DISTANCE = 100217
GLU_MAP1_TRIM_2 = 100210
GLU_MAP1_TRIM_3 = 100211
GLU_OUTLINE_POLYGON = 100240
GLU_OUTLINE_PATCH = 100241
GLU_NURBS_ERROR1 = 100251
GLU_NURBS_ERROR2 = 100252
GLU_NURBS_ERROR3 = 100253
GLU_NURBS_ERROR4 = 100254
GLU_NURBS_ERROR5 = 100255
GLU_NURBS_ERROR6 = 100256
GLU_NURBS_ERROR7 = 100257
GLU_NURBS_ERROR8 = 100258
GLU_NURBS_ERROR9 = 100259
GLU_NURBS_ERROR10 = 100260
GLU_NURBS_ERROR11 = 100261
GLU_NURBS_ERROR12 = 100262
GLU_NURBS_ERROR13 = 100263
GLU_NURBS_ERROR14 = 100264
GLU_NURBS_ERROR15 = 100265
GLU_NURBS_ERROR16 = 100266
GLU_NURBS_ERROR17 = 100267
GLU_NURBS_ERROR18 = 100268
GLU_NURBS_ERROR19 = 100269
GLU_NURBS_ERROR20 = 100270
GLU_NURBS_ERROR21 = 100271
GLU_NURBS_ERROR22 = 100272
GLU_NURBS_ERROR23 = 100273
GLU_NURBS_ERROR24 = 100274
GLU_NURBS_ERROR25 = 100275
GLU_NURBS_ERROR26 = 100276
GLU_NURBS_ERROR27 = 100277
GLU_NURBS_ERROR28 = 100278
GLU_NURBS_ERROR29 = 100279
GLU_NURBS_ERROR30 = 100280
GLU_NURBS_ERROR31 = 100281
GLU_NURBS_ERROR32 = 100282
GLU_NURBS_ERROR33 = 100283
GLU_NURBS_ERROR34 = 100284
GLU_NURBS_ERROR35 = 100285
GLU_NURBS_ERROR36 = 100286
GLU_NURBS_ERROR37 = 100287
GLU_CW = 100120
GLU_CCW = 100121
GLU_INTERIOR = 100122
GLU_EXTERIOR = 100123
GLU_UNKNOWN = 100124
GLU_BEGIN = GLU_TESS_BEGIN
GLU_VERTEX = GLU_TESS_VERTEX
GLU_END = GLU_TESS_END
GLU_ERROR = GLU_TESS_ERROR
GLU_EDGE_FLAG = GLU_TESS_EDGE_FLAG




import traceback
import string
import types
from shortcut import Shortcut
import cmd
from cmd import _cmd,lock,lock_attempt,unlock,QuietException, \
     _feedback,fb_module,fb_mask 
from cmd import is_string

boolean_type = 1
int_type     = 2
float_type   = 3
float3_type  = 4

class SettingIndex:
   bonding_vdw_cutoff    =0
   min_mesh_spacing      =1
   dot_density           =2
   dot_mode              =3
   solvent_radius        =4
   sel_counter           =5
   bg_rgb                =6
   ambient               =7
   direct                =8
   reflect               =9
   light                =10
   power                =11
   antialias            =12
   cavity_cull          =13
   ambient_scale        =14
   single_image         =15
   movie_delay          =16
   ribbon_power         =17
   ribbon_power_b       =18
   ribbon_sampling      =19
   ribbon_radius        =20
   stick_radius         =21
   hash_max             =22
   orthoscopic          =23  # note, ortho still works...
   spec_reflect         =24
   spec_power           =25
   sweep_angle          =26
   sweep_speed          =27
   dot_hydrogens        =28
   dot_radius           =29
   ray_trace_frames     =30
   cache_frames         =31
   trim_dots            =32
   cull_spheres         =33
   test1                =34
   test2                =35
   surface_best         =36
   surface_normal       =37
   surface_quality      =38
   surface_proximity    =39
   normal_workaround    =40 
   stereo_angle         =41 
   stereo_shift         =42
   line_smooth          =43
   line_width           =44
   half_bonds           =45
   stick_quality        =46
   stick_overlap        =47
   stick_nub            =48
   all_states           =49
   pickable             =50
   auto_show_lines      =51
   idle_delay           =52
   no_idle              =53
   fast_idle            =54
   slow_idle            =55
   rock_delay           =56
   dist_counter         =57
   dash_length          =58
   dash_gap             =59
   auto_zoom            =60
   overlay              =61
   text                 =62
   button_mode          =63 
   valence              =64
   nonbonded_size       =65
   label_color          =66
   ray_trace_fog        =67
   spheroid_scale       =68
   ray_trace_fog_start  =69
   spheroid_smooth      =70
   spheroid_fill        =71
   auto_show_nonbonded  =72
   cache_display        =73
   mesh_radius          =74
   backface_cull        =75
   gamma                =76
   dot_width            =77
   auto_show_selections =78
   auto_hide_selections =79
   selection_width      =80
   selection_overlay    =81
   static_singletons    =82
   max_triangles        =83
   depth_cue            =84
   specular             =85
   shininess            =86
   sphere_quality       =87
   fog                  =88
   isomesh_auto_state   =89
   mesh_width           =90
   cartoon_sampling     =91
   cartoon_loop_radius  =92
   cartoon_loop_quality =93
   cartoon_power        =94
   cartoon_power_b      =95
   cartoon_rect_length  =96
   cartoon_rect_width   =97
   internal_gui_width   =98
   internal_gui         =99
   cartoon_oval_length  =100
   cartoon_oval_width   =101
   cartoon_oval_quality =102
   cartoon_tube_radius  =103
   cartoon_tube_quality =104
   cartoon_debug        =105
   ribbon_width         =106
   dash_width           =107
   dash_radius          =108
   cgo_ray_width_scale  =109
   line_radius          =110
   cartoon_round_helices     =111
   cartoon_refine_normals    =112
   cartoon_flat_sheets       =113
   cartoon_smooth_loops      =114
   cartoon_dumbbell_length   =115
   cartoon_dumbbell_width    =116
   cartoon_dumbbell_radius   =117
   cartoon_fancy_helices     =118
   cartoon_fancy_sheets      =119
   ignore_pdb_segi       =120
   ribbon_throw          =121
   cartoon_throw         =122
   cartoon_refine        =123
   cartoon_refine_tips   =124
   cartoon_discrete_colors   =125
   normalize_ccp4_maps   =126
   surface_poor          =127
   internal_feedback     =128
   cgo_line_width        =129
   cgo_line_radius       =130
   logging               =131
   robust_logs           =132
   log_box_selections    =133
   log_conformations     =134
   valence_default       =135
   surface_miserable     =136
   ray_opaque_background =137
   transparency          =138
   ray_texture           =139
   ray_texture_settings  =140
   suspend_updates       =141
   full_screen           =142
   surface_mode          =143
   surface_color         =144
   mesh_mode             =145
   mesh_color            =146
   auto_indicate_flags   =147
   surface_debug         =148
   ray_improve_shadows   =149
   smooth_color_triangle =150
   ray_default_renderer  =151
   field_of_view         =152
   reflect_power         =153
   preserve_chempy_ids   =154
   sphere_scale          =155
   two_sided_lighting    =156
   secondary_structure   =157
   auto_remove_hydrogens =158
   raise_exceptions      =159
   stop_on_exceptions    =160
   sculpting             =161
   auto_sculpt           =162
   sculpt_vdw_scale      =163
   sculpt_vdw_scale14    =164
   sculpt_vdw_weight     =165
   sculpt_vdw_weight14   =166
   sculpt_bond_weight    =167
   sculpt_angl_weight    =168
   sculpt_pyra_weight    =169
   sculpt_plan_weight    =170
   sculpting_cycles      =171
   sphere_transparency   =172
   sphere_color          =173
   sculpt_field_mask     =174
   sculpt_hb_overlap     =175
   sculpt_hb_overlap_base=176
   legacy_vdw_radii      =177
   sculpt_memory         =178
   connect_mode          =179
   cartoon_cylindrical_helices = 180
   cartoon_helix_radius  =181
   connect_cutoff        =182
#   save_pdb_ss           =183
   sculpt_line_weight    =184
   fit_iterations        =185
   fit_tolerance         =186
   batch_prefix          =187
   stereo_mode           =188
   cgo_sphere_quality    =189
   
setting_sc = Shortcut(SettingIndex.__dict__.keys())
   
index_list = []
name_dict = {}
name_list = SettingIndex.__dict__.keys()
name_list = filter(lambda x:x[0]!='_',name_list)
name_list.sort()
tmp_list = map(lambda x:(getattr(SettingIndex,x),x),name_list)
for a in tmp_list:
   name_dict[a[0]]=a[1]
   index_list.append(a[0])
   
boolean_dict = {
   "true" : 1,
   "false" : 0,
   "on"   : 1,
   "off"  : 0,
   "1"    : 1,
   "0"    : 0,
   }
   
boolean_sc = Shortcut(name_list)
  
def _get_index(name):
   # this may be called from C, so don't raise any exceptions...
   result = setting_sc.interpret(name)
   if type(result)==types.StringType:
      if hasattr(SettingIndex,result):
         return getattr(SettingIndex,result)
      else:
         return -1
   else:
      return -1

def _get_name(index): # likewise, this can be called from C so no exceptions
   if name_dict.has_key(index):
      return name_dict[index]
   else:
      return ""


def get_index_list():
   return index_list

def get_name_list():
   return name_list

###### API functions

def set(name,value,selection='',state=0,quiet=1,updates=1,log=0):
   '''
DESCRIPTION
  
   "set" changes one of the PyMOL state variables,
      
USAGE
 
   set name, value [,object-or-selection [,state ]]

   set name = value      # (DEPRECATED)

   WARNING: object and state specific settings are not yet fully
     implemented -- look for them in version 0.51.
 
PYMOL API
 
   cmd.set ( string name, string value,
             string selection='', int state=0,
             int quiet=0, int updates=1 )

NOTES

   The default behavior (with a blank selection) changes the global
   settings database.  If the selection is 'all', then the settings
   database in all individual objects will be changed.  Likewise, for
   a given object, if state is zero, then the object database will be
   modified.  Otherwise, the settings database for the indicated state
   within the object will be modified.

   If a selection is provided, then all objects in the selection will
   be affected. 
   
   '''
   r = None
   if log:
      cmd.log("set %s=%s\n"%(name,value))
   index = _get_index(str(name))
   if(index<0):
      print "Error: unknown setting '%s'."%name
      raise QuietException
   else:
      success = 0
      try:
         lock()
         type = _cmd.get_setting_tuple(int(index),str(""),int(-1))[0]
         if type==None:
            print "Error: unable to get setting type."
            raise QuietException
         try:
            if type==1: # boolean
               v = (boolean_dict[
                      boolean_sc.auto_err(
                         str(value),"boolean")],)
            elif type==2: # int
               v = (int(value),)
            elif type==3: # float
               v = (float(value),)
            elif type==4: # float3 - some legacy handling req.
               if is_string(value):
                  if not ',' in value:
                     v = string.split(value)
                  else:
                     v = eval(value)
               else:
                  v = value
               v = (float(v[0]),float(v[1]),float(v[2]))
            elif type==5: # color
               v = (str(value),)
            elif type==6: # string
               v = (str(value),)
               
            v = (type,v)
            r = _cmd.set(int(index),v,
                         string.strip(str(selection)),
                         int(state)-1,int(quiet),
                         int(updates))
         except:
            if(_feedback(fb_module.cmd,fb_mask.debugging)):
               traceback.print_exc()
               raise QuietException
            print "Error: unable to read setting value."
      finally:
         unlock()
   return r

def get_setting(name,object='',state=0): # INTERNAL
   r = None
   if is_string(name):
      i = _get_index(name)
   else:
      i = int(name)
   if i<0:
      print "Error: unknown setting"
      raise QuietException
   try:
      lock()
      r = _cmd.get_setting_tuple(i,str(object),int(state)-1)
      typ = r[0]
      if typ<3:
         r = int(r[1][0])
      elif typ<4:
         r = r[1][0]
      elif typ<5:
         r = r[1]
   finally:
      unlock()
   return r

def get_setting_tuple(name,object='',state=0): # INTERNAL
   r = None
   if is_string(name):      i = _get_index(name)
   else:
      i = int(name)
   if i<0:
      print "Error: unknown setting"
      raise QuietException
   try:
      lock()
      r = _cmd.get_setting_tuple(i,str(object),int(state)-1)
   finally:
      unlock()
   return r

def get_setting_text(name,object='',state=0):  # INTERNAL
   r = None
   if is_string(name):
      i = _get_index(name)
   else:
      i = int(name)
   if i<0:
      print "Error: unknown setting"
      raise QuietException
   try:
      lock()
      r = _cmd.get_setting_text(i,str(object),int(state)-1)
   finally:
      unlock()
   return r

def get_setting_updates(): # INTERNAL
   r = []
   if lock_attempt():
      try:
         r = _cmd.get_setting_updates()
      finally:
         unlock()
   return r


def get_setting_legacy(name): # INTERNAL, DEPRECATED
   r = None
   try:
      lock()
      r = _cmd.get_setting(name)
   finally:
      unlock()
   return r




import copy

class Neighbor:

   def __init__(self,vect_list,spacing):
      self.voxel = {}
      self.neighbor = None
      self.spacing = spacing
      c = 0
      voxel = self.voxel
      for v in vect_list:
         k = self.address(v)
         if not voxel.has_key(k):
            voxel[k] = [c]
         else:
            voxel[k].append(c)
         c = c + 1

   def optimize(self):
      self.neighbor = {}
      voxel = self.voxel
      for k in voxel.keys():
         lst = self.neighbor[k]
         for a in (k[0]-1,k[0],k[0]+1):
            for b in (k[1]-1,k[1],k[1]+1):
               for c in (k[2]-1,k[2],k[2]+1):
                  k2 = (a,b,c)
                  if voxel.has_key(k2):
                     lst.extend(voxel[k2])

   def address(self,vect):
      return (int(vect[0]/self.spacing),
              int(vect[1]/self.spacing),
              int(vect[2]/self.spacing))

   def get_voxel(self,vect):
      k = self.address(vect)
      if self.voxel.has_key(k):
         return self.voxel[k]
      else:
         return []
      
   def get_neighbors(self,vect):
      if self.neighbor:
         k = self.address(vect)
         if self.neighbor.has_key(k):
            return self.neighbor[k]
         else:
            return []
      else:
         voxel = self.voxel
         k = self.address(vect)
         lst = []
         for a in (k[0]-1,k[0],k[0]+1):
            for b in (k[1]-1,k[1],k[1]+1):
               for c in (k[2]-1,k[2],k[2]+1):
                  k2 = (a,b,c)
                  if voxel.has_key(k2):
                     lst.extend(voxel[k2])
         return lst
         
         



#A* -------------------------------------------------------------------
#B* This file contains source code for the PyMOL computer program
#C* copyright 1998-2000 by Warren Lyford Delano of DeLano Scientific. 
#D* -------------------------------------------------------------------
#E* It is unlawful to modify or remove this copyright notice.
#F* -------------------------------------------------------------------
#G* Please see the accompanying LICENSE file for further information. 
#H* -------------------------------------------------------------------
#I* Additional authors of this source file include:
#-* 
#-* 
#-*
#Z* -------------------------------------------------------------------

import thread
import string
import types

import pymol
import selector

import cmd
from cmd import _cmd,lock,unlock,Shortcut,QuietException,_raising, \
     _feedback,fb_module,fb_mask, \
     repres,repres_sc, \
     toggle_dict,toggle_sc,stereo_dict,stereo_sc


view_dict = {}
view_sc = Shortcut(['store','recall'])
view_dict_sc = Shortcut([])

def zoom(selection="all",buffer=0.0,state=0):
   '''
DESCRIPTION
  
   "zoom" scales and translates the window and the origin to cover the
   atom selection.
      
USAGE
 
   zoom object-or-selection [,buffer [, state]]
   zoom (selection) [,buffer [, state]]
 
PYMOL API

   cmd.zoom( string object-or-selection [,float buffer=0.0 [, int state=0 ]] )

NOTES

   state = 0 (default) use all coordinate states
   state = -1 use only coordinates for the current state
   state > 0  use coordinates for a specific state
   
SEE ALSO

   origin, orient
   '''
   # preprocess selection
   selection = selector.process(selection)
   #   
   try:
      lock()   
      r = _cmd.zoom(str(selection),float(buffer),int(state)-1)
   finally:
      unlock()
   return r


clip_action_sc = Shortcut([ 'near','far','move','slab' ])

def clip(mode,offset):
   '''
DESCRIPTION
  
   "clip" alterss the near and far clipping planes according to 
      
USAGE
  
   clip {near|far|move|slab}, distance
 
EXAMPLES
 
   clip near, -5  # moves near plane away from you by 5 A
   clip far, 10   # moves far plane towards you by 10 A
   clip slab, 20  # sets slab thickness to 20 A
   clip move, -5  # moves the slab away from you by 5 A
   
PYMOL API

   cmd.clip( string mode, float distance )

SEE ALSO

   zoom, reset
   '''
   mode = clip_action_sc.auto_err(str(mode),'mode')
   try:
      lock()   
      r = _cmd.clip(str(mode),float(offset))
   finally:
      unlock()
   return r

def origin(selection="(all)",object=None,position=None,state=0):
   '''
DESCRIPTION
  
   "origin" sets the center of rotation about a selection.
   If an object name is specified, it can be used to set
   the center of rotation for the object's TTT matrix.
      
USAGE
 
   origin selection [, object [,position, [, state]]]
   origin (selection)
   origin position=[1.0,2.0,3.0]

PYMOL API
 
   cmd.origin( string object-or-selection )

NOTES

   state = 0 (default) use all coordinate states
   state = -1 use only coordinates for the current state
   state > 0  use coordinates for a specific state

SEE ALSO

   zoom, orient, reset
   '''
   #'
   # preprocess selection
   selection = selector.process(selection)
   #   
   try:
      lock()
      if object==None: object=''
      if position==None: position=(0.0,0.0,0.0)
      else:
         if cmd.is_string(position):
            position = eval(position)
         selection = ''
      r = _cmd.origin(selection,str(object),
                      (float(position[0]),
                       float(position[1]),
                       float(position[2])
                       ),int(state)-1)
   finally:
      unlock()
   return r

def orient(selection="(all)",state=0):
   '''
DESCRIPTION
  
   "orient" aligns the principal components of the atoms in the
   selection with the XYZ axes.  The function is similar to the
   orient command in X-PLOR.
      
USAGE
 
   orient object-or-selection [, state]
   orient (selection)
 
PYMOL API
 
   cmd.orient( string object-or-selection [, state = 0] )

NOTES

   state = 0 (default) use all coordinate states
   state = -1 use only coordinates for the current state
   state > 0  use coordinates for a specific state

SEE ALSO

   zoom, origin, reset
   '''
   # preprocess selection
   selection = selector.process(selection)
   #   
   try:
      lock()
      r = _cmd.orient(selection,int(state)-1)
   finally:
      unlock()
   return r

   
def move(axis,distance):
   '''
DESCRIPTION
  
   "move" translates the world about one of the three primary axes.
      
USAGE
  
   move axis,distance
 
EXAMPLES
 
   move x,3
   move y,-1
    
PYMOL API
 
   cmd.move( string axis, float distance )

SEE ALSO

   turn
   '''
   try:
      lock()   
      r = _cmd.move(str(axis),float(distance))
   finally:
      unlock()
   return r

def enable(name='all'):
   '''
DESCRIPTION
  
   "enable" enable display of an object and all currently visible representations.
   
USAGE
 
   enable name
   enable all

   name = object or selection name
   
PYMOL API
 
   cmd.enable( string object-name )
 
EXAMPLE
 
   enable my_object

SEE ALSO

   show, hide, disable
   '''
   try:
      lock()   
      r = _cmd.onoff(str(name),1);
   finally:
      unlock()
   return r

def disable(name='all'):
   '''
DESCRIPTION
  
   "disable" disables display of an object and all currently visible
   representations.
   
USAGE
 
   disable name
   disable all 

   "name" is the name of an object or a named selection
   
PYMOL API
 
   cmd.disable( string name ) 
 
EXAMPLE
 
   disable my_object

SEE ALSO

   show, hide, enable   
   '''
   try:
      lock()   
      r = _cmd.onoff(str(name),0);
   finally:
      unlock()
   return r


def show(representation="",selection=""):
   '''
DESCRIPTION
  
   "show" turns on atom and bond representations.
 
   The available representations are:
    
      lines     spheres   mesh      ribbon     cartoon
      sticks    dots      surface   labels
      nonbonded nb_spheres 
   
USAGE
 
   show
   show reprentation [,object]
   show reprentation [,(selection)]
   show (selection)
   
PYMOL API
 
   cmd.show( string representation="", string selection="" )
 
EXAMPLES
 
   show lines,(name ca or name c or name n)
   show ribbon
 
NOTES

   "selection" can be an object name
   "show" alone will turn on lines for all bonds.

SEE ALSO

   hide, enable, disable
   '''
   r=1
   try:
      lock()
      if (representation=="") and (selection==""):
         r = _cmd.showhide("(all)",repres['lines'],1); # show lines by default       
      elif (representation!="") and (selection!=""):
         rep = representation
         rep = repres_sc.auto_err(rep,'representation')
         repn = repres[rep];
         # preprocess selection 
         selection = selector.process(selection)
         #   
         r = _cmd.showhide(str(selection),int(repn),1);
      elif representation=='all':
         r = _cmd.showhide("all",repres['lines'],1); # show lines by default 
      elif (representation[0:1]=='(') or (string.find(representation,'/')>=0):
         # preprocess selection
         selection = selector.process(representation)
         #                  
         r = _cmd.showhide(str(selection),repres['lines'],1);
      else: # selection==""
         rep = representation
         rep = repres_sc.auto_err(rep,'representation')
         repn = repres[rep];
         r = _cmd.showhide("all",int(repn),1);
   finally:
      unlock()
   return r

def hide(representation="",selection=""):
   '''
DESCRIPTION
  
   "hide" turns of atom and bond representations.
 
   The available representations are:
    
      lines     spheres   mesh      ribbon     cartoon
      sticks    dots      surface   labels
      nonbonded nb_spheres
   
USAGE
 
   hide reprentation [,object]
   hide reprentation [,(selection)]
   hide (selection)
   
PYMOL API
 
   cmd.hide( string representation="", string selection="")
 
EXAMPLES
 
   hide lines,all
   hide ribbon

SEE ALSO

   show, enable, disable
   '''
   r = 1
   try:
      lock()
      if (representation=="") and (selection==""):
         r = _cmd.showhide("@",0,0);      
      elif (representation!="") and (selection!=""):
         rep = representation
         rep = repres_sc.auto_err(rep,'representation')
         repn = repres[rep];
         selection = selector.process(selection)
         r = _cmd.showhide(str(selection),int(repn),0);
      elif (representation=='all'):
         r = _cmd.showhide("@",0,0);
      elif (representation[0:1]=='(') or (string.find(representation,'/')>=0):
         selection = selector.process(representation)
         r = _cmd.showhide(str(selection),-1,0);
      else: # selection == ""
         rep = representation
         rep = repres_sc.auto_err(rep,'representation')
         repn = repres[rep];
         r = _cmd.showhide("all",int(repn),0);
   finally:
      unlock()
   return r


def get_view(output=1):
   '''
DESCRIPTION
 
   "get_view" returns and optionally prints out the current view
   information in a format which can be embedded into a command
   script and used in subsequent calls to "set_view"
 
USAGE

   get_view
   
PYMOL API

   cmd.get_view(output=1)  
 
API USAGE

   cmd.get_view(0) # zero option suppresses output
'''
   
   r = None
   try:
      lock()
      r = _cmd.get_view()
   finally:
      unlock()
   if len(r):
      if cmd.get_setting_legacy("logging")!=0.0:
         print " get_view: matrix written to log file."
         cmd.log("_ set_view (\\\n","cmd.set_view((\\\n")
         cmd.log("_  %14.9f, %14.9f, %14.9f,\\\n"%r[0:3]  , "  %14.9f, %14.9f, %14.9f,\\\n"%r[0:3])
         cmd.log("_  %14.9f, %14.9f, %14.9f,\\\n"%r[4:7]  , "  %14.9f, %14.9f, %14.9f,\\\n"%r[4:7])
         cmd.log("_  %14.9f, %14.9f, %14.9f,\\\n"%r[8:11] , "  %14.9f, %14.9f, %14.9f,\\\n"%r[8:11])
         cmd.log("_  %14.9f, %14.9f, %14.9f,\\\n"%r[16:19], "  %14.9f, %14.9f, %14.9f,\\\n"%r[16:19])
         cmd.log("_  %14.9f, %14.9f, %14.9f,\\\n"%r[19:22], "  %14.9f, %14.9f, %14.9f,\\\n"%r[19:22]) 
         cmd.log("_  %14.9f, %14.9f, %14.9f )\n"%r[22:25] , "  %14.9f, %14.9f, %14.9f ))\n"%r[22:25])
         if output<2: # suppress if we have a log file open
            output=0
      if output:
         print "### cut below here and paste into script ###"
         print "set_view (\\"
         print "  %14.9f, %14.9f, %14.9f,\\"%r[0:3]
         print "  %14.9f, %14.9f, %14.9f,\\"%r[4:7]
         print "  %14.9f, %14.9f, %14.9f,\\"%r[8:11]
         print "  %14.9f, %14.9f, %14.9f,\\"%r[16:19]
         print "  %14.9f, %14.9f, %14.9f,\\"%r[19:22]
         print "  %14.9f, %14.9f, %14.9f )"%r[22:25]
         print "### cut above here and paste into script ###"
      
   r = r[0:3]+r[4:7]+r[8:11]+r[16:25]
   return r

def set_view(view):
   '''
DESCRIPTION
 
   "set_view" sets viewing information for the current scene,
   including the rotation matrix, position, origin of rotation,
   clipping planes, and the orthoscopic flag.
 
USAGE

   set_view (...)  where ... is 18 floating point numbers
   
PYMOL API

   cmd.set_view(string-or-sequence view)  
 
'''
   
   r = None
   if cmd.is_string(view):
      try:
         view = eval(view)
      except:
         print "Error: bad view argument; should be a sequence of 18 floats."
         raise QuietException
   if len(view)!=18:
      print "Error: bad view argument; should be a sequence of 18 floats."
      raise QuietException
   else:
      try:
         lock()
         r = _cmd.set_view((
            float(view[ 0]),float(view[ 1]),float(view[ 2]),0.0,
            float(view[ 3]),float(view[ 4]),float(view[ 5]),0.0,
            float(view[ 6]),float(view[ 7]),float(view[ 8]),0.0,
            0.0,0.0,0.0,1.0,
            float(view[ 9]),float(view[10]),float(view[11]),
            float(view[12]),float(view[13]),float(view[14]),
            float(view[15]),float(view[16]),float(view[17])))
      finally:
         unlock()
   return r

def view(key,action='recall'):
   '''
DESCRIPTION
 
   "view" makes it possible to save and restore viewpoints on a given
   scene within a single session.
 
USAGE
 
   view key[,action]
   view ?

   key can be any string
   action should be 'store' or 'recall' (default: 'recall')
   
PYMOL API

   cmd.view(string key,string action)
   
EXAMPLES

   view 0,store
   view 0

SEE ALSO

   get_view
   '''
   if key=='?':
      print " view: stored views:"
      parsing.dump_str_list(view_dict.keys())
   else:
      action = view_sc.auto_err(action,'action')
      if action=='recall':
         key = view_dict_sc.auto_err(key,'view')
         set_view(view_dict[key])
         if _feedback(fb_module.scene,fb_mask.blather): # redundant
            print " view: recalled."
      elif action=='store':
         view_dict_sc.append(key)
         view_dict[key]=cmd.get_view(0)
         if _feedback(fb_module.scene,fb_mask.actions):
            print " view: view stored as '%s'."%key

def stereo(state='on'):
   '''
DESCRIPTION
 
   "stereo" activates or deactives stereo mode.  Currently only
   high-end stereo graphics are supported on the SGI (stereo in a
   window).
 
USAGE
 
   stereo on
   stereo off
   stereo swap
   stereo crosseye 
   stereo quadbuffer

NOTES

   quadbuffer is the default stereo mode if hardware stereo is available
   otherwise, crosseye is the default
   
PYMOL API

   cmd.stereo(string state="on")
   '''
   state = stereo_dict[stereo_sc.auto_err(str(state),'state')]
   r = None
   try:
      lock()
      if state>1:
         if state==2: # cross-eye
            cmd.set("stereo_mode","2",quiet=1)
         elif state==3: # quad
            cmd.set("stereo_mode","1",quiet=1)
         state=1
      if not _cmd.stereo(state):
         print "Error: Selected stereo mode is not available."
         if _raising(): raise QuietException
   finally:
      unlock();
   return r


def turn(axis,angle):
   '''
DESCRIPTION
  
   "turn" rotates the world about one of the three primary axes
      
USAGE
  
   turn axis, angle
 
EXAMPLES
 
   turn x,90
   turn y,45
    
PYMOL API
 
   cmd.turn( string axis, float angle )

SEE ALSO

   move
   '''
   try:
      lock()
      r = _cmd.turn(str(axis),float(angle))
   finally:
      unlock()
   return r


def full_screen(toggle=1):
   '''
DESCRIPTION

   "full_screen" enables or disables PyMOL's full_screen mode.  This
   is only functions well on PC's.

USAGE

   full_screen on
   full_screen off
   
'''
   toggle = toggle_dict[toggle_sc.auto_err(str(toggle),'toggle')]
   if thread.get_ident() == pymol.glutThread:
      try: 
         lock()
         r = _cmd.full_screen(int(toggle))
      finally:
         unlock()
   else:
      try:
         lock()
         r = _cmd.do("_cmd.full_screen(%d)"%int(toggle))
      finally:
         unlock()
   return r


def rock(mode=-1):
   '''
DESCRIPTION
  
   "rock" toggles Y axis rocking.
 
USAGE
 
   rock
 
PYMOL API
  
   cmd.rock()
   '''
   try:
      lock()   
      r = _cmd.rock(int(mode))
   finally:
      unlock()
   return r

def label(selection="(all)",expression=""):
   '''
DESCRIPTION
 
   "label" labels one or more atoms properties over a selection using
   the python evaluator with a separate name space for each atom.  The
   symbols defined in the name space are:
 
      name, resn, resi, chain, q, b, segi, type (ATOM,HETATM) 
      formal_charge, partial_charge, numeric_type, text_type
   
   All strings in the expression must be explicitly quoted.  This
   operation typically takes several seconds per thousand atoms
   altered.

   To clear labels, simply omit the expression or set it to ''.
   
USAGE

   label (selection),expression
   
EXAMPLES
  
   label (chain A),chain
   label (n;ca),"%s-%s" % (resn,resi)
   label (resi 200),"%1.3f" % partial_charge
   '''
   # preprocess selection
   selection = selector.process(selection)
   #
   r = 1
   try:
      lock()
      if len(str(expression))==0:
         r= _cmd.label("("+str(selection)+")",'')
      else:
         r = _cmd.label("("+str(selection)+")",'label='+str(expression))
   finally:
      unlock()   
   return r


def viewport(width=-1,height=-1):
   '''
DESCRIPTION
  
   "viewport" changes the size of the viewing port (and thus the size
   of all png files subsequently output)
      
USAGE
 
   viewport width, height
 
PYMOL API
  
   cmd.viewport(int width, int height)
   '''
   r = None
   if not cmd.is_glut_thread():
      cmd.do("viewport %d,%d"%(int(width),int(height)))
   else:
      try:
         lock()
         r = _cmd.viewport(int(width),int(height))
      finally:
         unlock()
   return r


def bg_color(color="black"):
   '''
DESCRIPTION
 
   "bg_color" sets the background color

USAGE

   bg_color [color]

PYMOL API

   cmd.color(string color="black")
   
   '''
   r = None
   color = cmd._interpret_color(color)
   try:
      lock()
      r = _cmd.bg_color(str(color))
   finally:
      unlock()
   if not r:
      if _raising(): raise QuietException
   return r

cartoon_dict = {
   'skip'        : -1,
   'automatic'   : 0,
   'loop'        : 1,
   'rectangle'   : 2,
   'oval'        : 3,
   'tube'        : 4,
   'arrow'       : 5,
   'dumbbell'    : 6,   
}

cartoon_sc = Shortcut(cartoon_dict.keys())

def cartoon(type,selection="(all)"):
   '''
DESCRIPTION
  
   "cartoon" changes the default cartoon for a set of atoms.
      
USAGE
 
   cartoon type, (selection)

   type = skip | automatic | loop | rectangle | oval | tube | arrow | dumbbell

PYMOL API
  
   cmd.cartoon(string type, string selection )

EXAMPLES

   cartoon rectangle,(chain A)
   cartoon skip,(resi 145:156)
   
NOTES

   the "automatic" mode utilizes ribbons according to the
   information in the PDB HELIX and SHEET records.

'''
   # preprocess selection
   selection = selector.process(selection)
   #
   type = cartoon_dict[cartoon_sc.auto_err(str(type),'type')];
   r = 1
   try:
      lock()   
      r = _cmd.cartoon("("+str(selection)+")",int(type))
   finally:
      unlock()
   return r

def ray(width=0,height=0,renderer=-1):
   '''
DESCRIPTION
  
   "ray" creates a ray-traced image of the current frame. This
   can take some time (up to several minutes, depending on image
   complexity).
      
USAGE
 
   ray [width,height [,renderer ]]

EXAMPLES

   ray
   ray 1024,768
   ray renderer=0
   
PYMOL API
  
   cmd.ray(int width,int height,int renderer=-1,string prefix)

NOTES

   renderer = -1 is default (use value in ray_default_renderer)
   renderer =  0 uses PyMOL's internal renderer
   renderer =  1 uses PovRay's renderer.  This is Unix-only
      and you must have "x-povray" in your path.  It utilizes two
      two temporary files: "tmp_pymol.pov" and "tmp_pymol.png".
      
SEE ALSO

   "help faster" for optimization tips with the builtin renderer.
   "help povray" for how to use PovRay instead of PyMOL's built-in
      ray-tracing engine.
   
   '''
   try:
      lock()   
      r = _cmd.render(int(width),int(height),int(renderer))
   finally:
      unlock()
   return r

def refresh():
   '''
DESCRIPTION
  
   "refresh" causes the scene to be refresh as soon as it is safe to
   do so.

USAGE

   refresh

PYMOL API
 
   cmd.refresh()

SEE ALSO

   rebuild
   '''
   if thread.get_ident() == pymol.glutThread:
      r = _cmd.refresh_now()
   else:
      try:
         lock()
         r = _cmd.do("cmd._refresh()")
      finally:
         unlock()
   return r

def reset(object=''):
   '''
DESCRIPTION
  
   "reset" restores the rotation matrix to identity, sets the origin
   to the center of mass (approx.) and zooms the window and clipping
   planes to cover all objects.
   
USAGE
 
   reset 
 
PYMOL API
 
   cmd.reset ( )
   '''
   try:
      lock()   
      r = _cmd.reset(0,str(object))
   finally:
      unlock()
   return r


def dirty(): # OBSOLETE?
   try:
      lock()
      r = _cmd.dirty()
   finally:
      unlock()
   return r

def meter_reset():
   '''
DESCRIPTION
  
   "meter_reset" resets the frames per secound counter
   
USAGE

   meter_reset
   '''
   try:
      lock()   
      r = _cmd.reset_rate()
   finally:
      unlock()
   return r

def load_png(filename,movie=1,quiet=0):
   r=None
   try:
      lock()   
      r = _cmd.load_png(str(filename),int(movie),int(quiet))
   finally:
      unlock()
   return r


def rebuild(selection='all',representation='everything'):
   '''
DESCRIPTION

   "rebuild" forces PyMOL to recreate geometric objects in
   case any of them have gone out of sync.

USAGE
   
   rebuild [selection [, representation ]]
   
PYMOL API

   cmd.rebuild(string selection = 'all', string representation = 'everything')

SEE ALSO

   refresh
'''
   r = 1
   selection = selector.process(selection)
   representation = repres_sc.auto_err(representation,'representation')
   repn = repres[representation];
   try:
      lock()
      r = _cmd.rebuild(selection,repn)
   finally:
      unlock()

def recolor(selection='all',representation='everything'):
   '''
DESCRIPTION

   "rebuild" forces PyMOL to reapply colors to geometric objects in
   case any of them have gone out of sync.

USAGE
   
   recolor [selection [, representation ]]
   
PYMOL API

   cmd.recolor(string selection = 'all', string representation = 'everything')

SEE ALSO

   recolor
'''
   r = 1
   selection = selector.process(selection)
   representation = repres_sc.auto_err(representation,'representation')
   repn = repres[representation];
   try:
      lock()
      r = _cmd.recolor(selection,repn)
   finally:
      unlock()
         
def color(color,selection="(all)"):
   '''
DESCRIPTION
  
   "color" changes the color of an object or an atom selection.
   
USAGE
 
   color color-name
   color color-name, object-name
   color color-name, (selection)
 
PYMOL API
  
   cmd.color( string color, string color-name )
 
EXAMPLES 
 
   color yellow, (name C*)
   '''
   # preprocess selection
   selection = selector.process(selection)
   color = cmd._interpret_color(color)
   #
   try:
      lock()
      r = _cmd.color(str(color),str(selection),0)
   finally:
      unlock()
   if not r:
      if _raising(): raise QuietException
   return r


def set_color(name,rgb):
   '''
DESCRIPTION
  
   "set_color" defines a new color with color indices (0.0-1.0)
   
USAGE

   set_color name, [ red-float, green-float, blue-float ]
   
   set_color name = [ red-float, green-float, blue-float ]
     # (DEPRECATED)
     
PYMOL API
  
   cmd.set_color( string name, float-list rgb )
 
EXAMPLES 
 
   set_color red = [ 1.0, 0.0, 0.0 ]
   '''
   r = 1
   if cmd.is_string(rgb):
      rgb = eval(rgb)
   if not (isinstance(rgb,types.ListType) or isinstance(rgb,types.TupleType)):
      print "Error: color specification must be a list such as [ 1.0, 0.0, 0.0 ]"
   elif len(rgb)!=3:
      print "Error: color specification must be a list such as [ 1.0, 0.0, 0.0 ]"
   else:
      try:
         lock()

         if len(rgb)==3:
            r = _cmd.colordef(str(name),float(rgb[0]),float(rgb[1]),float(rgb[2]))
            cmd._invalidate_color_sc()
         else:
            print "Error: invalid color."
      finally:
         unlock()
   return r




from chempy import tinker,io,feedback
from chempy.tinker import keyword
import copy
import string

class State:

   def __init__(self):
      if feedback['verbose']:
         print ' '+str(self.__class__)+': created.'
      self.prefix = 'tinker'
      self.params = tinker.params_path+'chempy.prm'

      self.default = {}
      self.default['timestep'] = 1.0
      self.default['temperature'] = 300.0

      self.keywords = {}
      self.keywords['verbose'] = ''
      self.keywords['cutoff'] = 8.0      
      self.keywords['randomseed'] = 1234567890

      self.mapping = None
      self.echo = 0
      self.model = None
      self.reset_fragile()
      self.counter = 0

   def reset_fragile(self):
      self.restart = None
      self.frames = None
      
   def __write_keys(self,list):
      f=open(self.prefix+"_inp.key",'w')
      f.writelines(list)
      f.close()

   def analyze(self,kw=None,summary=1):
      if feedback['actions']:
         print ' '+str(self.__class__)+': starting energy run...'
      io.xyz.toFile(self.model,self.prefix+"_inp.xyz",
                    mapping = self.mapping)
      kw_list = [ "parameters "+self.params+"\n" ]
      for a in self.keywords.keys():
         kw_list.append("%s %s\n" % ( a,str(self.keywords[a])))
      kw_list.extend(keyword.get_partial_charge(self.model))
      if kw:
         kw_list.extend(kw)
      self.__write_keys(kw_list)
      tinker.do('analyze',
                self.prefix+"_inp",
                self.prefix+"_run",
                self.prefix+"_out",
                [ tinker.prefix,
                  'E',],
                capture = 1 + self.echo )
      if summary:
# get energy information from output file
         f = open(self.prefix+"_out.out")
         self.summary = []
         flag = 0
         while 1:
            lin = f.readline()
            if not lin: break
            if not flag:
               if lin[0:25]==' Total Potential Energy :':
                  self.summary.append([
                     string.strip(lin[0:23]),
                     float(lin[25:49])])
                  flag = 1
            else:
               tok = string.split(string.strip(lin))
               if len(tok):
                  if(tok[0]!='Energy'):
                     self.summary.append([
                        string.strip(lin[0:23]),
                        float(lin[25:49]),
                        int(lin[49:64])])
         f.close()
      else:
         self.summary = None
      if len(self.summary):
          self.energy = self.summary[0][1]
      else:
          self.energy = None

   def minimize(self,gradient=0.1,max_iter=100,
                kw=None,summary=1):
      if feedback['actions']:
         print ' '+str(self.__class__)+': starting minimization run...'
      io.xyz.toFile(self.model,self.prefix+"_inp.xyz")
      self.counter = self.counter + max_iter
      kw_list = [ "parameters "+self.params+"\n" ]
      for a in self.keywords.keys():
         kw_list.append("%s %s\n" % ( a,str(self.keywords[a])))
      kw_list.extend(keyword.get_partial_charge(self.model))
      if kw:
         kw_list.extend(kw)
      kw_list.append("overwrite\n")
      kw_list.append("maxiter %d\n" % max_iter)
      self.__write_keys(kw_list)
      tinker.do('minimize',
                self.prefix+"_inp",
                self.prefix+"_run",
                self.prefix+"_out",
                [ tinker.prefix,
                  str(gradient),],
                capture = 1 + self.echo )
      io.xyz.updateFromFile(self.model,self.prefix+"_out.xyz")
      self.restart = None
      if summary:
# get energy information from output file
         f = open(self.prefix+"_out.out")
         step = 1
         flag = 0
         self.summary = []
         while 1:
            lin = f.readline()
            if not lin: break
            if not flag:
               if (lin[0:9]==' CG Iter ' or
                   lin[0:9]==' QN Iter '):
                  f.readline() # skip blank
                  flag = 1
            else:
               try:
                  if int(lin[0:6])==step:
                     step = step + 1
                     self.summary.append([
                        int(lin[0:6]),
                        float(lin[6:19]),
                        float(lin[19:30]),
                        float(lin[30:41]),
                        float(lin[41:51]),
                        float(lin[51:60]),
                        int(lin[60:67]),
                        string.strip(lin[67:])])
               except ValueError:
                  pass
         f.close()
      else:
         self.summary = None
      if len(self.summary):
          self.energy = self.summary[-1][1]
      else:
          self.energy = None
         
   def dynamics(self,steps=100,
                timestep=None,
                interval=None,
                temperature=None,
                kw=None,summary=1):
      if feedback['actions']:
         print ' '+str(self.__class__)+': starting dynamics run...'
      io.xyz.toFile(self.model,self.prefix+"_inp.xyz")
      self.counter = self.counter + steps * timestep
# if restart is available, then use it.
      if self.restart:
         f = open(self.prefix+"_inp.dyn","w")
         f.writelines(self.restart)
         f.close()
         if feedback['actions']:
            print ' '+str(self.__class__)+': using restart information...'
#         
      kw_list = [ "parameters "+self.params+"\n" ]
      for a in self.keywords.keys():
         kw_list.append("%s %s\n" % ( a,str(self.keywords[a])))
      kw_list.extend(keyword.get_partial_charge(self.model))
      if kw:
         kw_list.extend(kw)
      kw_list.append("archive\n")
      self.__write_keys(kw_list)
      if not timestep:
         timestep = self.default['timestep']
      if not temperature:
         temperature = self.default['temperature']
      if not interval:
         interval = (timestep*steps)/1000.0
      else:
         interval = interval/1000.0
      tinker.do('dynamic',
                self.prefix+"_inp",
                self.prefix+"_run",
                self.prefix+"_out",
                [ tinker.prefix,
                  str(steps),
                  str(timestep),
                  str(interval),
                  str(temperature)],
                capture = 1 + self.echo )
      self.frames = io.arc.fromFile(self.prefix+"_out.arc")
      if len(self.frames):
         c = 0
         atm = self.model.atom
         for a in self.frames[-1]:
            atm[c].coord = copy.deepcopy(a)
            c = c + 1
# get restart information
      f = open(self.prefix+"_out.dyn")
      self.restart = f.readlines()
      f.close()
# get energy information from output file
      if summary:
         f = open(self.prefix+"_out.out")
         step = 1
         flag = 0
         self.summary = []
         while 1:
            lin = f.readline()
            if not lin: break
            if not flag:
               if lin[0:12]=='    MD Step ':
                  f.readline() # skip blank
                  flag = 1
            else:
               try:
                  if int(lin[0:10])==step:
                     step = step + 1
                     self.summary.append([
                        int(lin[0:10]),
                        float(lin[10:24]),
                        float(lin[24:38]),
                        float(lin[38:50]),
                        float(lin[50:61])])
               except ValueError:
                  pass
         f.close()
      else:
         self.summary = None
      if len(self.summary):
         self.energy = self.summary[-1][1]
      else:
         self.energy = None

      
   def load_model(self,a):
      if feedback['verbose']:
         print ' '+str(self.__class__)+': new model loaded.'
      self.model = a
      self.reset_fragile()









import copy

def __list_sum(the_list):
   return reduce(lambda x,y:x+y,the_list)

def combine_fragments(*arg,**kw):
   '''
   merge_fragments(target,source1,source2,source3,...)
   
   WARNING: This is not how you *should* combine fragments,
   rather this is just a convenient kludge for getting
   something remotely realistic in a big hurry.
   
   NOTE: atom names must be unique throughout -- especially
   including the "capping" atoms used during the ab-initio
   calculations (you can use PyMOL to rename these if they
   happen to coincide).
'''
   
   if kw.has_key('net_charge'):
      net_charge = kw['net_charge']
   else:
      net_charge = None
   if len(arg)<1:
      raise TypeError('invalid arguments')
   dst = copy.deepcopy(arg[0])
   frg_lst = arg[1:]
   n_frg = len(frg_lst)
   n_dst_atm = len(dst.atom)

   # create dictionary/indices for shared atoms
   
   n_tot = 0
   dst_dict = {}
   for a in dst.atom:
      sig = a.name
      dst_dict[sig] = n_tot
      n_tot = n_tot + 1

   # create fragment membership list for destination atoms
   
   members = []
   for a in range(n_tot):
      members.append([])
   
   # create indices for unique atoms

   for fragment in frg_lst:
      c = 0
      for a in fragment.atom:
         sig = a.name
         if dst_dict.has_key(sig):
            a.chg_index = dst_dict[sig]
            members[a.chg_index].append(fragment)
         else:
            # find an attached atom that *is* in the destination structure
            attached_index = None
            for b in fragment.bond:
               attached = None
               if b.index[0]==c:
                  attached = fragment.atom[b.index[1]]
               elif b.index[1]==c:
                  attached = fragment.atom[b.index[0]]
               if attached!=None:
                  if dst_dict.has_key(attached.name):
                     attached_index = dst_dict[attached.name]
                     break
            a.chg_index = n_tot
            if attached_index!=None:
               a.attached_index = attached_index
            n_tot = n_tot + 1
         c = c + 1
            
   # create array for charges
   
   chg = []
   for a in range(n_frg):
      chg.append([0.0] * n_tot)
   
   # now load and count measurements

   cnt = [0] * n_tot
   c = 0
   for fragment in frg_lst:
      for a in fragment.atom:
         index = a.chg_index
         if (index < n_dst_atm):
            chg[c][index] = chg[c][index] + a.partial_charge
            cnt[index] = cnt[index] + 1
         else:
            aindex = a.attached_index
            chg[c][aindex] = chg[c][aindex] + a.partial_charge
      c = c + 1

   # now compute average charges for each destination atom

   avg = []
   for index in range(n_dst_atm):
      if cnt[index]:
         tmp_lst = []
         for a in chg:
            tmp_lst.append(a[index])
         avg.append(__list_sum(tmp_lst)/cnt[index])
      else:
         avg.append(0.0)
      
   # correct total charge

   chg_sum = __list_sum(avg)

   print "chg_sum",chg_sum
   if net_charge != None:
      chg_diff = net_charge - chg_sum
      chg_adjust = chg_diff / n_dst_atm
   else:
      chg_adjust = 0.0
      
   c = 0
   for a in dst.atom:
      a.partial_charge = avg[c] + chg_adjust
      c = c + 1

   return dst





from pymol import cmd
from pymol import util

class Demo: # stateful class for doing effective demonstrations

   def __init__(self):
      self.last = None
      
   def __call__(self,name):
      if self.last:
         self.last(cleanup=1)
      if hasattr(self,name):
         self.last = getattr(self,name)
         self.last()
                 
   def rep(self,cleanup=0):
      if not cleanup:
         cmd.set("suspend_updates",1,quiet=1)
         cmd.disable()
         cmd.do("cd $PYMOL_PATH")
         cmd.delete("pept")
         cmd.delete("pept_dist")
         cmd.load("test/dat/pept.pdb")
         cmd.show("sticks","(pept and not i;5:7)")
         cmd.show("surface","(pept and i;5,6)")
         cmd.show("mesh","(pept and i;1,11,12,13)")
         cmd.show("spheres","(pept and i;2,12,9,4 and not n;c,n,o,ca)")
         cmd.show("dots","(i;8)")
         cmd.dist("pept_dist","(pept and i;1&n;OD2)","(pept and i;13&n;OG1)")
         cmd.set("dot_width","2");
         cmd.set("suspend_updates",0,quiet=1)
      else:
         cmd.delete("pept")
         cmd.delete("pept_dist")
      
   def raster3d(self,cleanup=0):
      if not cleanup:
         cmd.disable()
         cmd.do("cd $PYMOL_PATH")
         cmd.delete("cgo1")
         cmd.delete("cgo2")
         cmd.do("cd $PYMOL_PATH")
         cmd.load("test/dat/pept.r3d","cgo1")
         cmd.load("test/dat/3al1.r3d","cgo2")
         cmd.zoom()
      else:
         cmd.delete("cgo1")
         cmd.delete("cgo2")

   def cgo(self,cleanup=0):
      if not cleanup:
         cmd.disable()
         cmd.do("cd $PYMOL_PATH")
         cmd.do("run examples/devel/cgo03.py")
      else:
         cmd.delete("cgo03")

   def anime(self,cleanup=0):
      if not cleanup:
         cmd.disable()
         cmd.delete("arg")
         cmd.fragment("arg")
         cmd.zoom("arg",2)
         cmd.show("sticks","arg")
         cmd.feedback('dis','sel','res')
         for a in xrange(1,181):
            cmd.set("suspend_updates",1,quiet=1)
            cmd.edit("(arg and n;cd)","(arg and n;cg)")
            cmd.torsion("6")
            cmd.unpick()
            cmd.edit("(arg and n;cb)","(arg and n;ca)")
            cmd.torsion("2")
            cmd.unpick()
            cmd.set("suspend_updates",0,quiet=1)         
            cmd.refresh()
         cmd.feedback('ena','sel','res')
      else:
         cmd.delete("arg")

   def cartoon(self,cleanup=0):
      if not cleanup:
         cmd.set("suspend_updates",1,quiet=1)
         cmd.disable()
         cmd.delete("1tii")      
         cmd.load("$PYMOL_PATH/test/dat/1tii.pdb")
         cmd.hide("(1tii)")
         cmd.show("cartoon","1tii")
         cmd.zoom("1tii")
         util.color_chains("1tii")
         cmd.set("suspend_updates",0,quiet=1)
         cmd.refresh()
      else:
         cmd.delete("1tii")

   def trans(self,cleanup=0):
      if not cleanup:
         cmd.set("suspend_updates",1,quiet=1)
         cmd.disable()
         cmd.delete("trans")
         cmd.load("$PYMOL_PATH/test/dat/pept.pdb","trans")
         cmd.hide("(tran)")
         cmd.show("surface","trans")
         cmd.show("sticks","trans")
         cmd.set("surface_color","white","trans")
         cmd.set("transparency",0.5,"trans")
         cmd.zoom("trans")
         cmd.set("suspend_updates",0,quiet=1)
         cmd.refresh()
      else:
         cmd.delete("trans")

   def ray(self,cleanup=0):
      if not cleanup:
         cmd.set("suspend_updates",1,quiet=1)
         cmd.disable()
         cmd.delete("ray")
         cmd.load("$PYMOL_PATH/test/dat/il2.pdb","ray")
         cmd.remove("(ray and hydro)")
         cmd.hide("lines","ray")
         cmd.show("spheres","ray")
         cmd.orient("ray")
         cmd.turn("x",90)
         util.ray_shadows('heavy')
         cmd.set("suspend_updates",0,quiet=1)
         cmd.refresh()
         cmd.do("ray")
      else:
         cmd.delete("ray")

   def sculpt(self,cleanup=0):
      if not cleanup:
         cmd.set("suspend_updates",1,quiet=1)
         cmd.disable()
         cmd.delete("sculpt")
         cmd.load("$PYMOL_PATH/test/dat/pept.pdb","sculpt")
         cmd.show("spheres","sculpt")
         cmd.set("sphere_scale","0.5","sculpt")
         cmd.set("sphere_transparency","0.5","sculpt")
         cmd.set("sphere_color","grey","sculpt")
         cmd.sculpt_activate("sculpt")
         cmd.set("auto_sculpt",1)
         cmd.set("sculpting",1)
         cmd.do("edit_mode")
         cmd.set("valence","0.05")
         cmd.set("suspend_updates",0,quiet=0)
      else:
         cmd.set("sculpting",0)
         cmd.set("auto_sculpt",0)
         cmd.delete("sculpt")



from chempy import Storage,Atom,feedback
from chempy.models import Indexed,Connected
import string
import copy

generics =  {
   'CT' : 1,
   'C'  : 2,
   'CA' : 3,
   'CM' : 4,
   'CC' : 5,
   'CV' : 6,
   'CW' : 7,
   'CR' : 8,
   'CB' : 9,
   'C*' : 10,
   'CN' : 11,
   'CK' : 12,
   'CQ' : 13,
   'N'  : 14,
   'NA' : 15,
   'NB' : 16,
   'NC' : 17,
   'N*' : 18,
   'N2' : 19,
   'N3' : 20,
   'OW' : 21,
   'OH' : 22,
   'OS' : 23,
   'O'  : 24,
   'O2' : 25,
   'S'  : 26,
   'SH' : 27,
   'P'  : 28,
   'H'  : 29,
   'HW' : 30,
   'HO' : 31,
   'HS' : 32,
   'HA' : 33,
   'HC' : 34,
   'H1' : 35,
   'H2' : 36,
   'H3' : 37,
   'HP' : 38,
   'H4' : 39,
   'H5' : 40,
}

class XYZ(Storage):
   
#------------------------------------------------------------------------------
   def toList(self,model,mapping=None):

      conn = copy.deepcopy(model)
      conn = conn.convert_to_connected()
      
      list = []

      if len(model.atom):
         if not model.atom[0].has('numeric_type'):
            if not mapping:
               mapping = generics
               if feedback['warnings']:
                  print ' '+str(self.__class__)+': no numeric atom types found, using defaults.'
         list.append("%6d\n" % conn.nAtom)
         c = 0
         for a in conn.atom:
            if mapping:
               n_type = mapping[a.text_type]
            else:
               n_type = a.numeric_type
            if n_type<0:
               print str(self.__class__)+\
                     '-WARNING: negative numeric type (%d) for atom %d'% (n_type,c)
            st = "%6d  %-3s%12.6f%12.6f%12.6f%6d" % (
               c+1,a.text_type,a.coord[0],a.coord[1],a.coord[2],int(n_type))
            for b in conn.bond[c]:
               idx = b.index[0]
               if idx == c:
                  idx = b.index[1]
               st = st + "%6d" % (idx+1)
            st = st + "\n"
            list.append(st)
            c = c + 1
      return(list)

#----------------------------------------------------------------------------
   def updateFromList(self,model,list):

      c = 0
      for a in list[1:]:
         model.atom[c].coord = [ float(a[11:23]),float(a[23:35]),float(a[35:47])]
         c = c + 1




#A* -------------------------------------------------------------------
#B* This file contains source code for the PyMOL computer program
#C* copyright 1998-2000 by Warren Lyford Delano of DeLano Scientific. 
#D* -------------------------------------------------------------------
#E* It is unlawful to modify or remove this copyright notice.
#F* -------------------------------------------------------------------
#G* Please see the accompanying LICENSE file for further information. 
#H* -------------------------------------------------------------------
#I* Additional authors of this source file include:
#-* 
#-* 
#-*
#Z* -------------------------------------------------------------------


import math
import selector

import cmd
from cmd import _cmd,lock,unlock,Shortcut,QuietException
from chempy import cpv

def sculpt_purge():
   '''
   undocumented
'''
   r = 0
   try:
      lock()
      r = _cmd.sculpt_purge()
   finally:
      unlock()
   return r

def sculpt_deactivate(object):
   '''
   undocumented
'''
   r = 0
   try:
      lock()
      r = _cmd.sculpt_deactivate(str(object))
   finally:
      unlock()
   return r
   
def sculpt_activate(object,state=0):
   '''
undocumented.
'''
   r = 0
   try:
      lock()
      r = _cmd.sculpt_activate(str(object),int(state)-1)
   finally:
      unlock()
   return r

def sculpt_iterate(object,state=0,cycles=10):
   '''
undocumented.
'''
   r = 0
   try:
      lock()
      r = _cmd.sculpt_iterate(str(object),int(state)-1,int(cycles))
   finally:
      unlock()
   return r

def smooth(cycles,selection,first,last,window):
   '''
undocumented
   '''
   r = 0
   selection = selector.process(selection)   
   try:
      lock()
      r = _cmd.smooth(str(selection),int(cycles),
                      int(first)-1,int(last)-1,int(window))
   finally:
      unlock()
   return r
   
def set_geometry(selection,geometry,valence):
   '''
DESCRIPTION
  
   "set_geometry" changes PyMOL's assumptions about the proper valence
   and geometry of the picked atom.
      
USAGE
 
   set_geometry geometry, valence
 
PYMOL API
  
   cmd.set_geometry(int geometry,int valence )

NOTES

   Immature functionality. See code for details.

SEE ALSO

   remove, attach, fuse, bond, unbond
'''
   r = 0
   # preprocess selection
   selection = selector.process(selection)
   try:
      lock()
      r = _cmd.set_geometry(str(selection),int(geometry),int(valence))
   finally:
      unlock()
   return r

def undo():
   '''
DESCRIPTION

   "undo" restores the previous conformation of the object currently
   being edited.
   
USAGE
 
   undo

SEE ALSO

   redo, push_undo
'''
   r = None
   try:
      lock()
      r = _cmd.undo(-1)
   finally:
      unlock()
   return r

def push_undo(selection,state=0):
   '''
DESCRIPTION

   "push_undo" stores the currently conformations of objects in the
   selection onto their individual kill rings.
   
USAGE
 
   push_undo (all)

SEE ALSO

   undo, redo
'''
   # preprocess selections
   selection = selector.process(selection)
   #
   r = None
   try:
      lock()
      r = _cmd.push_undo("("+str(selection)+")",int(state)-1)
   finally:
      unlock()
   return r

def redo():
   '''
DESCRIPTION

   "redo" reapplies the conformational change of the object currently
   being edited.
   
USAGE
 
   redo

SEE ALSO

   undo, push_undo
'''
   try:
      lock()
      _cmd.undo(1)
   finally:
      unlock()


def bond(atom1="(lb)",atom2="(rb)",order=1,edit=1):
   '''
DESCRIPTION
 
   "bond" creates a new bond between two selections, each of
   which should contain one atom.
 
USAGE

   bond [atom1,atom2 [,order]]
   
PYMOL API

   cmd.bond(string atom1, string atom2)
   
NOTES

   The atoms must both be within the same object.
   
   The default behavior is to create a bond between the (lb) and (rb)
   selections.

SEE ALSO

   unbond, fuse, attach, replace, remove_picked
'''
   # preprocess selections
   atom1 = selector.process(atom1)
   atom2 = selector.process(atom2)
   try:
      lock()
      r = _cmd.bond(atom1,atom2,int(order),1)
      if r and edit:
         _cmd.edit(str(atom1),str(atom2),'','',0)
   finally:
      unlock()
   return r

def invert(selection1="(lb)",selection2="(rb)"):
   '''
DESCRIPTION

   "invert" inverts the stereo-chemistry of the atom currently picked
   for editing (pk1).  Two additional atom selections must be provided
   in order to indicate which atoms remain stationary during the
   inversion process.

USAGE

   invert (selection1),(selection2)

PYMOL API

   cmd.api( string selection1="(lb)", string selection2="(lb)" )
   
NOTE

   The invert function is usually bound to CTRL-E in editing mode.

   The default selections are (lb) and (rb), meaning that you can pick
   the atom to invert with CTRL-middle click and then pick the
   stationary atoms with CTRL-SHIFT/left-click and CTRL-SHIFT/right-
   click, then hit CTRL-E to invert the atom.

'''
   # preprocess selections
   selection1 = selector.process(selection1)
   selection2 = selector.process(selection2)
   #
   try:
      lock()
      r = _cmd.invert(str(selection1),str(selection2),0)
   finally:
      unlock()
   return r

def unbond(atom1="(lb)",atom2="(rb)"):
   '''
DESCRIPTION
 
   "unbond" removes all bonds between two selections.

USAGE

   unbond atom1,atom2
   
PYMOL API

   cmd.unbond(selection atom1="(lb)",selection atom2="(rb)")
   
SEE ALSO

   bond, fuse, remove_picked, attach, detach, replace
 
'''
   # preprocess selections
   atom1 = selector.process(atom1)
   atom2 = selector.process(atom2)   
   try:
      lock()
      r = _cmd.bond(str(atom1),str(atom2),0,0)
   finally:
      unlock()
   return r

def remove(selection):
   '''
DESCRIPTION
  
   "remove" eleminates a selection of atoms from models.
      
USAGE
 
   remove (selection)
 
PYMOL API
  
   cmd.remove( string selection )
    
EXAMPLES
 
   remove ( resi 124 )

SEE ALSO

   delete
'''
   # preprocess selection
   selection = selector.process(selection)
   #   
   r = 1
   try:
      lock()   
      r = _cmd.remove("("+selection+")")
   finally:
      unlock()
   return r


def remove_picked(hydrogens=1):
   '''
DESCRIPTION
  
   "remove_picked" removes the atom or bond currently
   picked for editing. 
      
USAGE
 
   remove_picked [hydrogens]
 
PYMOL API
  
   cmd.remove_picked(integer hydrogens=1)

NOTES

   This function is usually connected to the
   DELETE key and "CTRL-D".
   
   By default, attached hydrogens will also be deleted unless
   hydrogen-flag is zero.

SEE ALSO

   attach, replace
'''
   r = 1
   try:
      lock()   
      r = _cmd.remove_picked(int(hydrogens))
   finally:
      unlock()
   return r


def cycle_valence(h_fill=1):
   '''
DESCRIPTION
  
   "cycle_valence" cycles the valence on the currently selected bond.
      
USAGE
 
   cycle_valence [ h_fill ]
 
PYMOL API
  
   cmd.cycle_valence(int h_fill)

EXAMPLES

   cycle_valence
   cycle_valence 0
   
NOTES

   If the h_fill flag is true, hydrogens will be added or removed to
   satisfy valence requirements.
   
   This function is usually connected to the DELETE key and "CTRL-W".

SEE ALSO

   remove_picked, attach, replace, fuse, h_fill
'''
   r = 1
   try:
      lock()   
      r = _cmd.cycle_valence()
   finally:
      unlock()
   if h_fill:
      globals()['h_fill']()
   return r


def attach(element,geometry,valence):
   '''
DESCRIPTION
  
   "attach" adds a single atom onto the picked atom.
      
USAGE
 
   attach element, geometry, valence
 
PYMOL API
  
   cmd.attach( element, geometry, valence )

NOTES

   Immature functionality.  See code for details.

'''
   r = 1
   try:
      lock()   
      r = _cmd.attach(str(element),int(geometry),int(valence))
   finally:
      unlock()
   return r


def fuse(selection1="(lb)",selection2="(rb)",mode=0):
   '''
DESCRIPTION
  
   "fuse" joins two objects into one by forming a bond.  A copy of
   the object containing the first atom is moved so as to form an
   approximately resonable bond with the second, and is then merged
   with the first object.
      
USAGE
 
   fuse (selection1), (selection2)
 
PYMOL API
  
   cmd.fuse( string selection1="(lb)", string selection2="(lb)" )

NOTES

   Each selection must include a single atom in each object.
   The atoms can both be hydrogens, in which case they are
   eliminated, or they can both be non-hydrogens, in which
   case a bond is formed between the two atoms.

SEE ALSO

   bond, unbond, attach, replace, fuse, remove_picked
'''
   # preprocess selections
   selection1 = selector.process(selection1)
   selection2 = selector.process(selection2)
   #   
   try:
      lock()
      r = _cmd.fuse(str(selection1),str(selection2),int(mode))
   finally:
      unlock()
   return r

def unpick(*arg):
   '''
DESCRIPTION

   "unpick" deletes the special "pk" atom selections (pk1, pk2, etc.)
   used in atom picking and molecular editing.

USAGE

   unpick

PYMOL API

   cmd.unpick()

SEE ALSO

   edit
   '''
   try:
      lock()   
      r = _cmd.unpick()
   finally:
      unlock()
   return r

      

def edit(selection1='',selection2='',selection3='',selection4='',pkresi=0):
   '''
DESCRIPTION
  
   "edit" picks an atom or bond for editing.
      
USAGE
 
   edit (selection) [ ,(selection) ]
 
PYMOL API
  
   cmd.edit( string selection  [ ,string selection ] )

NOTES

   If only one selection is provided, an atom is picked.
   If two selections are provided, the bond between them
   is picked (if one exists).

SEE ALSO

   unpick, remove_picked, cycle_valence, torsion
'''
   # preprocess selections
   selection1 = selector.process(selection1)
   selection2 = selector.process(selection2)
   selection3 = selector.process(selection3)
   selection4 = selector.process(selection4)
   #
   r = 1
   try:
      lock()   
      r = _cmd.edit(str(selection1),str(selection2),
                    str(selection3),str(selection4),int(pkresi))
   finally:
      unlock()
   return r


def torsion(angle):
   '''
DESCRIPTION
  
   "torsion" rotates the torsion on the bond currently
   picked for editing.  The rotated fragment will correspond
   to the first atom specified when picking the bond (or the
   nearest atom, if picked using the mouse).
      
USAGE
 
   torsion angle
 
PYMOL API
  
   cmd.torsion( float angle )

SEE ALSO

   edit, unpick, remove_picked, cycle_valence
'''
   try:
      lock()   
      r = _cmd.torsion(float(angle))
   finally:
      unlock()
   return r

def h_fill():
   '''
DESCRIPTION
  
   "h_fill" removes and replaces hydrogens on the atom
   or bond picked for editing.  
      
USAGE
 
   h_fill
 
PYMOL API
  
   cmd.h_fill()

NOTES
   
   This is useful for fixing hydrogens after changing
   bond valences.

SEE ALSO

   edit, cycle_valences, h_add
'''
   r = 1
   try:
      lock()   
      r = _cmd.h_fill()
   finally:
      unlock()
   return r

def h_add(selection="(all)"):
   '''
DESCRIPTION
  
   "h_add" uses a primitive algorithm to add hydrogens
   onto a molecule.
      
USAGE
 
   h_add (selection)
 
PYMOL API
  
   cmd.h_add( string selection="(all)" )

SEE ALSO

   h_fill
'''
   # preprocess selection
   selection = selector.process(selection)
   #   
   r = 1
   try:
      lock()   
      r = _cmd.h_add(selection)
   finally:
      unlock()
   return r
   


def sort(object=""):
   '''
DESCRIPTION
 
   "sort" reorders atoms in the structure.  It usually only necessary
   to run this routine after an "alter" command which has modified the
   names of atom properties.  Without an argument, sort will resort
   all atoms in all objects.

USAGE
 
   sort [object]

PYMOL API

   cmd.sort(string object)

SEE ALSO

   alter
'''
   try:
      lock()
      r = _cmd.sort(str(object))
   finally:
      unlock()
   return r


def replace(element,geometry,valence,h_fill=1):
   '''
DESCRIPTION
  
   "replace" replaces the picked atom with a new atom.
      
USAGE
 
   replace element, geometry, valence
 
PYMOL API
  
   cmd.replace(string element, int geometry,int valence )

NOTES

   Immature functionality. See code for details.

SEE ALSO

   remove, attach, fuse, bond, unbond
'''
   r = 1
   if not "pk1" in cmd.get_names("selections"):
      print " Error: you must first pick an atom to replace."
      raise QuietException
   try:
      if h_fill: # strip off existing hydrogens
         remove("((neighbor pk1) and elem h)")
      lock()
      r = _cmd.replace(str(element),int(geometry),int(valence))
   finally:
      unlock()
   return r

def rename(object,force=0):
   '''
DESCRIPTION
  
   "rename" creates new atom names which are unique within residues.
      
USAGE

   CURRENT
      rename object-name [ ,force ]
      
      force = 0 or 1 (default: 0)
      
   PROPOSED
      rename object-or-selection,force   

PYMOL API

   CURRENT
      cmd.rename( string object-name, int force )

   PROPOSED
      cmd.rename( string object-or-selection, int force )

NOTES

   To regerate only some atom names in a molecule, first clear them
   with an "alter (sele),name=''" commmand, then use "rename"

SEE ALSO

   alter
'''   
   try:
      lock()   
      r = _cmd.rename(str(object),int(force))
   finally:
      unlock()
   return r

def alter(selection,expression):
   '''
DESCRIPTION
 
   "alter" changes one or more atomic properties over a selection
   using the python evaluator with a separate name space for each
   atom.  The symbols defined in the name space are:
 
      name, resn, resi, chain, alt, elem, q, b, segi,
      type (ATOM,HETATM), partial_charge, formal_charge,
      text_type, numeric_type, ID
   
   All strings must be explicitly quoted.  This operation typically
   takes several seconds per thousand atoms altered.

   WARNING: You should always issue a "sort" command on an object
   after modifying any property which might affect canonical atom
   ordering (names, chains, etc.).  Failure to do so will confound
   subsequent "create" and "byres" operations.
   
USAGE
 
   alter (selection),expression
 
EXAMPLES
 
   alter (chain A),chain='B'
   alter (all),resi=str(int(resi)+100)
   sort
   
SEE ALSO

   alter_state, iterate, iterate_state, sort
   '''
   # preprocess selections
   selection = selector.process(selection)
   #
   try:
      lock()
      r = _cmd.alter("("+str(selection)+")",str(expression),0)
   finally:
      unlock()   
   return r


def iterate(selection,expression):
   '''
DESCRIPTION
 
   "iterate" iterates over an expression with a separate name space
   for each atom.  However, unlike the "alter" command, atomic
   properties can not be altered.  Thus, "iterate" is more efficient
   than "alter".

   It can be used to perform operations and aggregations using atomic
   selections, and store the results in any global object, such as the
   predefined "stored" object.

   The local namespace for "iterate" contains the following names

      name, resn, resi, chain, alt, elem,
      q, b, segi, and type (ATOM,HETATM),
      partial_charge, formal_charge,
      text_type, numeric_type, ID
 
   All strings in the expression must be explicitly quoted.  This
   operation typically takes a second per thousand atoms.
 
USAGE
 
   iterate (selection),expression
 
EXAMPLES

   stored.net_charge = 0
   iterate (all),stored.net_charge = stored.net_charge + partial_charge

   stored.names = []
   iterate (all),stored.names.append(name)

SEE ALSO

   iterate_state, atler, alter_state
   '''
   # preprocess selection
   selection = selector.process(selection)
   #
   try:
      lock()
      r = _cmd.alter("("+str(selection)+")",str(expression),1)
   finally:
      unlock()   
   return r

def alter_state(state,selection,expression):
   '''
DESCRIPTION
 
   "alter_state" changes the atomic coordinates of a particular state
   using the python evaluator with a separate name space for each
   atom.  The symbols defined in the name space are:
 
      x,y,z
 
USAGE
 
   alter_state state,(selection),expression
 
EXAMPLES
 
   alter 1,(all),x=x+5

SEE ALSO

   iterate_state, alter, iterate
   '''
   # preprocess selection
   selection = selector.process(selection)
   #
   state = int(state)
   if state<0: # hack -- need to replace with "alter_state_atom" command
      state = -state
      atomic_props = 1
   else:
      atomic_props = 0
   try:
      lock()
      r = _cmd.alter_state(int(state)-1,"("+str(selection)+")",str(expression),
                           0,int(atomic_props))
   finally:
      unlock()   
   return r

def iterate_state(state,selection,expression):
   '''
DESCRIPTION
 
   "iterate_state" is to "alter_state" as "iterate" is to "alter"
 
USAGE
 
   iterate_state state,(selection),expression
 
EXAMPLES

   stored.sum_x = 0.0
   iterate 1,(all),stored.sum_x = stored.sum_x + x

SEE ALSO

   iterate, alter, alter_state
   '''
   # preprocess selection
   selection = selector.process(selection)
   state = int(state)
   #
   if state<0: # hack -- need to replace with "alter_state_atom" command
      state = -state
      atomic_props = 1
   else:
      atomic_props = 0
   try:
      lock()
      r = _cmd.alter_state(int(state)-1,"("+str(selection)+")",
                           str(expression),1,int(atomic_props))
   finally:
      unlock()   
   return r

def translate(vector=[0.0,0.0,0.0],selection="all",state=0,camera=1,object=None):
   # INCOMPLETE:
   # needs to be modified so that selection can be something other
   # than an object name -- some work needs to be done on the
   # underlying C code
   r = 1
   if cmd.is_string(vector):
      vector = eval(vector)
   if not cmd.is_list(vector):
      print "Error: bad vector."
      raise QuietException
   else:
      vector = [float(vector[0]),float(vector[1]),float(vector[2])]
      selection = selector.process(selection)
      camera=int(camera)
      view = cmd.get_view(0)
      if camera:
         mat = [ view[0:3],view[3:6],view[6:9] ]
         shift = cpv.transform(mat,vector)
         print "hi",camera,camera and 1
      else:
         shift = vector
      print shift,camera
      if object==None:
         ttt = [1.0,0.0,0.0,0.0,
                0.0,1.0,0.0,0.0,
                0.0,0.0,1.0,0.0,
                shift[0],shift[1],shift[2],1.0]
         r=cmd.transform_object(selection,ttt)
      else:
         lock()
         ttt = [1.0, 0.0, 0.0, shift[0],
                0.0, 1.0, 0.0, shift[1],
                0.0, 0.0, 1.0, shift[2],
                0.0, 0.0, 0.0, 1.0]
         r=_cmd.combine_object_ttt(str(object),ttt)
         unlock()
         
   return r

def rotate(axis='x',angle=0.0,selection="all",state=0,camera=1,object=None):
   # INCOMPLETE:
   # needs to be modified so that selection can be something other
   # than an object name -- some work needs to be done on the
   # underlying C code
   r = 1
   if axis in ['x','X']:
      axis = [1.0,0.0,0.0]
   elif axis in ['y','Y']:
      axis = [0.0,1.0,0.0]
   elif axis in ['z','Z']:
      axis = [0.0,0.0,1.0]
   else:
      axis = eval(axis)
   if not cmd.is_list(axis):
      print "Error: bad axis."
      raise QuietException
   else:
      axis = [float(axis[0]),float(axis[1]),float(axis[2])]
      angle = math.pi*float(angle)/180.0
      view = cmd.get_view(0)
      camera=int(camera)
      if camera:
         vmat = [ view[0:3],view[3:6],view[6:9] ]
         axis = cpv.transform(vmat,axis)
      mat = cpv.rotation_matrix(angle,axis)
      if object==None:
         ttt = [mat[0][0],mat[1][0],mat[2][0],-view[12],
                mat[0][1],mat[1][1],mat[2][1],-view[13],
                mat[0][2],mat[1][2],mat[2][2],-view[14],
                view[12],view[13],view[14],view[15]]
         r=cmd.transform_object(selection,ttt)
      else:
         lock()
         ttt = [mat[0][0],mat[1][0],mat[2][0], 0.0,
                mat[0][1],mat[1][1],mat[2][1], 0.0,
                mat[0][2],mat[1][2],mat[2][2], 0.0,
                0.0      ,0.0      ,0.0      , 1.0]
         r=_cmd.combine_object_ttt(str(object),ttt)
         unlock()
   return r

def set_title(object,state,text):
   '''
DESCRIPTION

   "set_title" attaches a text string to the state of a particular
   object which can be displayed when the state is active.  This is
   useful for display the energies of a set of conformers.

USAGE
   
   set_title object,state,text

PYMOL API

   cmd.set_title(string object,int state,string text)

'''
   r = 1
   try:
      lock()
      r = _cmd.set_title(str(object),int(state)-1,str(text))
   finally:
      unlock()


def transform_object(name,matrix,state=0,log=0,sele=''):
   r = None
   try:
      lock()
      r = _cmd.transform_object(str(name),int(state)-1,list(matrix),int(log),str(sele))
   finally:
      unlock()
   return r

def translate_atom(sele1,v0,v1,v2,state=0,mode=0,log=0):
   r = None
   sele1 = selector.process(sele1)
   try:
      lock()
      r = _cmd.translate_atom(str(sele1),float(v0),float(v1),float(v2),int(state)-1,int(mode),int(log))
   finally:
      unlock()
   return r

def update(target,source,target_state=0,source_state=0):
   '''
DESCRIPTION
  
   "update" transfers coordinates from one selection to another.
USAGE
 
   update (target-selection),(source-selection)
 
EXAMPLES
 
   update target,(variant)

NOTES

   Currently, this applies across all pairs of states.  Fine
   control will be added later.

SEE ALSO

   load
'''
   
   a=target
   b=source
   # preprocess selections
   a = selector.process(a)
   b = selector.process(b)
   #
   if a[0]!='(': a="("+str(a)+")"
   if b[0]!='(': b="("+str(b)+")"   
   try:
      lock()   
      r = _cmd.update(str(a),str(b),int(target_state)-1,int(source_state)-1)
   finally:
      unlock()
   return r


def set_dihedral(atom1,atom2,atom3,atom4,angle,state=1):
   # preprocess selections
   atom1 = selector.process(atom1)
   atom2 = selector.process(atom2)
   atom3 = selector.process(atom3)
   atom4 = selector.process(atom4)
   #   
   try:
      lock()
      r = _cmd.set_dihe(str(atom1),str(atom2),str(atom3),str(atom4),float(angle),int(state)-1)
   finally:
      unlock()
   return r

def map_set_border(name,level=0.0):
   '''
DESCRIPTION
 
   "map_set_border" is a function (reqd by PDA) which allows you to set the
   level on the edge points of a map

USAGE

   map_set_border <name>,<level>

NOTES

   unsupported.
SEE ALSO

   load
   '''
   try:
      lock()
      r = _cmd.map_set_border(str(name),float(level))
   finally:
      unlock()
   return r


def protect(selection="(all)"):
   '''
DESCRIPTION

   "protect" protects a set of atoms from tranformations performed
   using the editing features.  This is most useful when you are
   modifying an internal portion of a chain or cycle and do not wish
   to affect the rest of the molecule.
   
USAGE

   protect (selection)

PYMOL API

   cmd.protect(string selection)
   
SEE ALSO

   deprotect, mask, unmask, mouse, editing
'''
   # preprocess selection
   selection = selector.process(selection)
   #   
   try:
      lock()   
      r = _cmd.protect("("+str(selection)+")",1)
   finally:
      unlock()
   return r

def deprotect(selection="(all)"):
   '''
DESCRIPTION

   "deprotect" reveres the effect of the "protect" command.

USAGE

   deprotect (selection)
   
PYMOL API

   cmd.deprotect(string selection="(all)")

SEE ALSO

   protect, mask, unmask, mouse, editing
'''
   # preprocess selection
   selection = selector.process(selection)
   #   
   try:
      lock()   
      r = _cmd.protect("("+str(selection)+")",0)
   finally:
      unlock()
   return r

flag_dict = {
# simulation 
   'focus'         : 0,
   'free'          : 1,
   'restrain'      : 2,
   'fix'           : 3,
   'exclude'       : 4,
# rendering
   'exfoliate'     : 24,
   'ignore'        : 25,
}

flag_sc = Shortcut(flag_dict.keys())

flag_action_dict = {
   'reset' : 0,
   'set'   : 1,
   'clear' : 2,
   }

flag_action_sc = Shortcut(flag_action_dict.keys())

   
def flag(flag,selection,action="reset",quiet=1):
   '''
DESCRIPTION
  
   "flag" sets the indicated flag for atoms in the selection and
    clears the indicated flag for atoms not in the selection.  This
    is primarily useful for passing selection information into
    Chempy models, which have a 32 bit attribute "flag" which holds
    this information.
   
USAGE

   flag flag, selection [ ,action ]
    
   flag flag = selection [ ,action ]      # (DEPRECATED)

   action can be:
     "reset" (default) sets flag on selection, clear it on other atoms 
     "set" sets the flag for selected atoms, leaves other atoms unchanged
     "clear" clear the flag for selected atoms, leaves other atoms unchanged

PYMOL API
  
   cmd.flag( int flag, string selection, string action="reset",
             int indicate=0)
   cmd.flag( string flag, string selection, string action="reset",
             int indicate=0)
 
EXAMPLES  
 
   flag free, (resi 45 x; 6)

NOTE

   If the 'auto_indicate_flags' setting is true, then PyMOL will automatically
   create a selection called "indicate" which contains all atoms with that flag
   after applying the command.
   
RESERVED FLAGS

   Flags 0-7 are reserved for molecular modeling */
      focus      0 = Atoms of Interest (i.e. a ligand in an active site)
      free       1 = Free Atoms (free to move subject to a force-field)
      restrain   2 = Restrained Atoms (typically harmonically contrained)
      fix        3 = Fixed Atoms (no movement allowed)
      ignore     4 = Atoms which should not be part of any simulation
   Flags 8-15 are free for end users to manipulate
   Flags 16-23 are reserved for external GUIs and linked applications
   Flags 24-31 are reserved for PyMOL internal usage
      exfoliate 24 = Remove surface from atoms when surfacing
      ignore    25 = Ignore atoms altogether when surfacing
   '''
   # preprocess selection
   new_flag = flag_sc.interpret(str(flag))
   if new_flag:
      if cmd.is_string(new_flag):
         flag = flag_dict[new_flag]
      else:
         flag_sc.auto_err(flag,'flag')
   # preprocess selection
   selection = selector.process(selection)
   action = flag_action_dict[flag_action_sc.auto_err(action,'action')]
   #      
   try:
      lock()
      r = _cmd.flag(int(flag),"("+str(selection)+")",
                    int(action),int(quiet))
   finally:
      unlock()
   return r

